from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class MeasureDimensionMap(Serialisable):
    tagname: str
    measureGroup: Incomplete
    dimension: Incomplete
    def __init__(self, measureGroup: Incomplete | None = ..., dimension: Incomplete | None = ...) -> None: ...

class MeasureGroup(Serialisable):
    tagname: str
    name: Incomplete
    caption: Incomplete
    def __init__(self, name: Incomplete | None = ..., caption: Incomplete | None = ...) -> None: ...

class PivotDimension(Serialisable):
    tagname: str
    measure: Incomplete
    name: Incomplete
    uniqueName: Incomplete
    caption: Incomplete
    def __init__(
        self,
        measure: Incomplete | None = ...,
        name: Incomplete | None = ...,
        uniqueName: Incomplete | None = ...,
        caption: Incomplete | None = ...,
    ) -> None: ...

class CalculatedMember(Serialisable):
    tagname: str
    name: Incomplete
    mdx: Incomplete
    memberName: Incomplete
    hierarchy: Incomplete
    parent: Incomplete
    solveOrder: Incomplete
    set: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        mdx: Incomplete | None = ...,
        memberName: Incomplete | None = ...,
        hierarchy: Incomplete | None = ...,
        parent: Incomplete | None = ...,
        solveOrder: Incomplete | None = ...,
        set: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class CalculatedItem(Serialisable):
    tagname: str
    field: Incomplete
    formula: Incomplete
    pivotArea: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        field: Incomplete | None = ...,
        formula: Incomplete | None = ...,
        pivotArea: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class ServerFormat(Serialisable):
    tagname: str
    culture: Incomplete
    format: Incomplete
    def __init__(self, culture: Incomplete | None = ..., format: Incomplete | None = ...) -> None: ...

class ServerFormatList(Serialisable):
    tagname: str
    serverFormat: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(self, count: Incomplete | None = ..., serverFormat: Incomplete | None = ...) -> None: ...
    @property
    def count(self): ...

class Query(Serialisable):
    tagname: str
    mdx: Incomplete
    tpls: Incomplete
    __elements__: Incomplete
    def __init__(self, mdx: Incomplete | None = ..., tpls: Incomplete | None = ...) -> None: ...

class QueryCache(Serialisable):
    tagname: str
    count: Incomplete
    query: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., query: Incomplete | None = ...) -> None: ...

class OLAPSet(Serialisable):
    tagname: str
    count: Incomplete
    maxRank: Incomplete
    setDefinition: Incomplete
    sortType: Incomplete
    queryFailed: Incomplete
    tpls: Incomplete
    sortByTuple: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        count: Incomplete | None = ...,
        maxRank: Incomplete | None = ...,
        setDefinition: Incomplete | None = ...,
        sortType: Incomplete | None = ...,
        queryFailed: Incomplete | None = ...,
        tpls: Incomplete | None = ...,
        sortByTuple: Incomplete | None = ...,
    ) -> None: ...

class OLAPSets(Serialisable):  # type: ignore[misc]
    count: Incomplete
    set: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., set: Incomplete | None = ...) -> None: ...

class PCDSDTCEntries(Serialisable):
    tagname: str
    count: Incomplete
    m: Incomplete
    n: Incomplete
    e: Incomplete
    s: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        count: Incomplete | None = ...,
        m: Incomplete | None = ...,
        n: Incomplete | None = ...,
        e: Incomplete | None = ...,
        s: Incomplete | None = ...,
    ) -> None: ...

class TupleCache(Serialisable):
    tagname: str
    entries: Incomplete
    sets: Incomplete
    queryCache: Incomplete
    serverFormats: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        entries: Incomplete | None = ...,
        sets: Incomplete | None = ...,
        queryCache: Incomplete | None = ...,
        serverFormats: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class PCDKPI(Serialisable):
    tagname: str
    uniqueName: Incomplete
    caption: Incomplete
    displayFolder: Incomplete
    measureGroup: Incomplete
    parent: Incomplete
    value: Incomplete
    goal: Incomplete
    status: Incomplete
    trend: Incomplete
    weight: Incomplete
    time: Incomplete
    def __init__(
        self,
        uniqueName: Incomplete | None = ...,
        caption: Incomplete | None = ...,
        displayFolder: Incomplete | None = ...,
        measureGroup: Incomplete | None = ...,
        parent: Incomplete | None = ...,
        value: Incomplete | None = ...,
        goal: Incomplete | None = ...,
        status: Incomplete | None = ...,
        trend: Incomplete | None = ...,
        weight: Incomplete | None = ...,
        time: Incomplete | None = ...,
    ) -> None: ...

class GroupMember(Serialisable):
    tagname: str
    uniqueName: Incomplete
    group: Incomplete
    def __init__(self, uniqueName: Incomplete | None = ..., group: Incomplete | None = ...) -> None: ...

class GroupMembers(Serialisable):  # type: ignore[misc]
    count: Incomplete
    groupMember: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., groupMember: Incomplete | None = ...) -> None: ...

class LevelGroup(Serialisable):
    tagname: str
    name: Incomplete
    uniqueName: Incomplete
    caption: Incomplete
    uniqueParent: Incomplete
    id: Incomplete
    groupMembers: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        uniqueName: Incomplete | None = ...,
        caption: Incomplete | None = ...,
        uniqueParent: Incomplete | None = ...,
        id: Incomplete | None = ...,
        groupMembers: Incomplete | None = ...,
    ) -> None: ...

class Groups(Serialisable):
    tagname: str
    count: Incomplete
    group: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., group: Incomplete | None = ...) -> None: ...

class GroupLevel(Serialisable):
    tagname: str
    uniqueName: Incomplete
    caption: Incomplete
    user: Incomplete
    customRollUp: Incomplete
    groups: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        uniqueName: Incomplete | None = ...,
        caption: Incomplete | None = ...,
        user: Incomplete | None = ...,
        customRollUp: Incomplete | None = ...,
        groups: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class GroupLevels(Serialisable):  # type: ignore[misc]
    count: Incomplete
    groupLevel: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., groupLevel: Incomplete | None = ...) -> None: ...

class FieldUsage(Serialisable):
    tagname: str
    x: Incomplete
    def __init__(self, x: Incomplete | None = ...) -> None: ...

class FieldsUsage(Serialisable):  # type: ignore[misc]
    count: Incomplete
    fieldUsage: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., fieldUsage: Incomplete | None = ...) -> None: ...

class CacheHierarchy(Serialisable):
    tagname: str
    uniqueName: Incomplete
    caption: Incomplete
    measure: Incomplete
    set: Incomplete
    parentSet: Incomplete
    iconSet: Incomplete
    attribute: Incomplete
    time: Incomplete
    keyAttribute: Incomplete
    defaultMemberUniqueName: Incomplete
    allUniqueName: Incomplete
    allCaption: Incomplete
    dimensionUniqueName: Incomplete
    displayFolder: Incomplete
    measureGroup: Incomplete
    measures: Incomplete
    count: Incomplete
    oneField: Incomplete
    memberValueDatatype: Incomplete
    unbalanced: Incomplete
    unbalancedGroup: Incomplete
    hidden: Incomplete
    fieldsUsage: Incomplete
    groupLevels: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        uniqueName: str = ...,
        caption: Incomplete | None = ...,
        measure: Incomplete | None = ...,
        set: Incomplete | None = ...,
        parentSet: Incomplete | None = ...,
        iconSet: int = ...,
        attribute: Incomplete | None = ...,
        time: Incomplete | None = ...,
        keyAttribute: Incomplete | None = ...,
        defaultMemberUniqueName: Incomplete | None = ...,
        allUniqueName: Incomplete | None = ...,
        allCaption: Incomplete | None = ...,
        dimensionUniqueName: Incomplete | None = ...,
        displayFolder: Incomplete | None = ...,
        measureGroup: Incomplete | None = ...,
        measures: Incomplete | None = ...,
        count: Incomplete | None = ...,
        oneField: Incomplete | None = ...,
        memberValueDatatype: Incomplete | None = ...,
        unbalanced: Incomplete | None = ...,
        unbalancedGroup: Incomplete | None = ...,
        hidden: Incomplete | None = ...,
        fieldsUsage: Incomplete | None = ...,
        groupLevels: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class GroupItems(Serialisable):
    tagname: str
    m: Incomplete
    n: Incomplete
    b: Incomplete
    e: Incomplete
    s: Incomplete
    d: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(self, count: Incomplete | None = ..., m=..., n=..., b=..., e=..., s=..., d=...) -> None: ...
    @property
    def count(self): ...

class DiscretePr(Serialisable):
    tagname: str
    count: Incomplete
    x: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., x: Incomplete | None = ...) -> None: ...

class RangePr(Serialisable):
    tagname: str
    autoStart: Incomplete
    autoEnd: Incomplete
    groupBy: Incomplete
    startNum: Incomplete
    endNum: Incomplete
    startDate: Incomplete
    endDate: Incomplete
    groupInterval: Incomplete
    def __init__(
        self,
        autoStart: bool = ...,
        autoEnd: bool = ...,
        groupBy: str = ...,
        startNum: Incomplete | None = ...,
        endNum: Incomplete | None = ...,
        startDate: Incomplete | None = ...,
        endDate: Incomplete | None = ...,
        groupInterval: int = ...,
    ) -> None: ...

class FieldGroup(Serialisable):
    tagname: str
    par: Incomplete
    base: Incomplete
    rangePr: Incomplete
    discretePr: Incomplete
    groupItems: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        par: Incomplete | None = ...,
        base: Incomplete | None = ...,
        rangePr: Incomplete | None = ...,
        discretePr: Incomplete | None = ...,
        groupItems: Incomplete | None = ...,
    ) -> None: ...

class SharedItems(Serialisable):
    tagname: str
    m: Incomplete
    n: Incomplete
    b: Incomplete
    e: Incomplete
    s: Incomplete
    d: Incomplete
    containsSemiMixedTypes: Incomplete
    containsNonDate: Incomplete
    containsDate: Incomplete
    containsString: Incomplete
    containsBlank: Incomplete
    containsMixedTypes: Incomplete
    containsNumber: Incomplete
    containsInteger: Incomplete
    minValue: Incomplete
    maxValue: Incomplete
    minDate: Incomplete
    maxDate: Incomplete
    longText: Incomplete
    __attrs__: Incomplete
    def __init__(
        self,
        _fields=...,
        containsSemiMixedTypes: Incomplete | None = ...,
        containsNonDate: Incomplete | None = ...,
        containsDate: Incomplete | None = ...,
        containsString: Incomplete | None = ...,
        containsBlank: Incomplete | None = ...,
        containsMixedTypes: Incomplete | None = ...,
        containsNumber: Incomplete | None = ...,
        containsInteger: Incomplete | None = ...,
        minValue: Incomplete | None = ...,
        maxValue: Incomplete | None = ...,
        minDate: Incomplete | None = ...,
        maxDate: Incomplete | None = ...,
        count: Incomplete | None = ...,
        longText: Incomplete | None = ...,
    ) -> None: ...
    @property
    def count(self): ...

class CacheField(Serialisable):
    tagname: str
    sharedItems: Incomplete
    fieldGroup: Incomplete
    mpMap: Incomplete
    extLst: Incomplete
    name: Incomplete
    caption: Incomplete
    propertyName: Incomplete
    serverField: Incomplete
    uniqueList: Incomplete
    numFmtId: Incomplete
    formula: Incomplete
    sqlType: Incomplete
    hierarchy: Incomplete
    level: Incomplete
    databaseField: Incomplete
    mappingCount: Incomplete
    memberPropertyField: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        sharedItems: Incomplete | None = ...,
        fieldGroup: Incomplete | None = ...,
        mpMap: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        name: Incomplete | None = ...,
        caption: Incomplete | None = ...,
        propertyName: Incomplete | None = ...,
        serverField: Incomplete | None = ...,
        uniqueList: bool = ...,
        numFmtId: Incomplete | None = ...,
        formula: Incomplete | None = ...,
        sqlType: int = ...,
        hierarchy: int = ...,
        level: int = ...,
        databaseField: bool = ...,
        mappingCount: Incomplete | None = ...,
        memberPropertyField: Incomplete | None = ...,
    ) -> None: ...

class RangeSet(Serialisable):
    tagname: str
    i1: Incomplete
    i2: Incomplete
    i3: Incomplete
    i4: Incomplete
    ref: Incomplete
    name: Incomplete
    sheet: Incomplete
    def __init__(
        self,
        i1: Incomplete | None = ...,
        i2: Incomplete | None = ...,
        i3: Incomplete | None = ...,
        i4: Incomplete | None = ...,
        ref: Incomplete | None = ...,
        name: Incomplete | None = ...,
        sheet: Incomplete | None = ...,
    ) -> None: ...

class PageItem(Serialisable):
    tagname: str
    name: Incomplete
    def __init__(self, name: Incomplete | None = ...) -> None: ...

class Page(Serialisable):
    tagname: str
    pageItem: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., pageItem: Incomplete | None = ...) -> None: ...
    @property
    def count(self): ...

class Consolidation(Serialisable):
    tagname: str
    autoPage: Incomplete
    pages: Incomplete
    rangeSets: Incomplete
    __elements__: Incomplete
    def __init__(self, autoPage: Incomplete | None = ..., pages=..., rangeSets=...) -> None: ...

class WorksheetSource(Serialisable):
    tagname: str
    ref: Incomplete
    name: Incomplete
    sheet: Incomplete
    def __init__(self, ref: Incomplete | None = ..., name: Incomplete | None = ..., sheet: Incomplete | None = ...) -> None: ...

class CacheSource(Serialisable):
    tagname: str
    type: Incomplete
    connectionId: Incomplete
    worksheetSource: Incomplete
    consolidation: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        type: Incomplete | None = ...,
        connectionId: Incomplete | None = ...,
        worksheetSource: Incomplete | None = ...,
        consolidation: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class CacheDefinition(Serialisable):
    mime_type: str
    rel_type: str
    records: Incomplete
    tagname: str
    invalid: Incomplete
    saveData: Incomplete
    refreshOnLoad: Incomplete
    optimizeMemory: Incomplete
    enableRefresh: Incomplete
    refreshedBy: Incomplete
    refreshedDate: Incomplete
    refreshedDateIso: Incomplete
    backgroundQuery: Incomplete
    missingItemsLimit: Incomplete
    createdVersion: Incomplete
    refreshedVersion: Incomplete
    minRefreshableVersion: Incomplete
    recordCount: Incomplete
    upgradeOnRefresh: Incomplete
    tupleCache: Incomplete
    supportSubquery: Incomplete
    supportAdvancedDrill: Incomplete
    cacheSource: Incomplete
    cacheFields: Incomplete
    cacheHierarchies: Incomplete
    kpis: Incomplete
    calculatedItems: Incomplete
    calculatedMembers: Incomplete
    dimensions: Incomplete
    measureGroups: Incomplete
    maps: Incomplete
    extLst: Incomplete
    id: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        invalid: Incomplete | None = ...,
        saveData: Incomplete | None = ...,
        refreshOnLoad: Incomplete | None = ...,
        optimizeMemory: Incomplete | None = ...,
        enableRefresh: Incomplete | None = ...,
        refreshedBy: Incomplete | None = ...,
        refreshedDate: Incomplete | None = ...,
        refreshedDateIso: Incomplete | None = ...,
        backgroundQuery: Incomplete | None = ...,
        missingItemsLimit: Incomplete | None = ...,
        createdVersion: Incomplete | None = ...,
        refreshedVersion: Incomplete | None = ...,
        minRefreshableVersion: Incomplete | None = ...,
        recordCount: Incomplete | None = ...,
        upgradeOnRefresh: Incomplete | None = ...,
        tupleCache: Incomplete | None = ...,
        supportSubquery: Incomplete | None = ...,
        supportAdvancedDrill: Incomplete | None = ...,
        cacheSource: Incomplete | None = ...,
        cacheFields=...,
        cacheHierarchies=...,
        kpis=...,
        calculatedItems=...,
        calculatedMembers=...,
        dimensions=...,
        measureGroups=...,
        maps=...,
        extLst: Incomplete | None = ...,
        id: Incomplete | None = ...,
    ) -> None: ...
    def to_tree(self): ...
    @property
    def path(self): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class RichText(Serialisable):
    tagname: str
    bodyPr: Incomplete
    properties: Incomplete
    lstStyle: Incomplete
    p: Incomplete
    paragraphs: Incomplete
    __elements__: Incomplete
    def __init__(
        self, bodyPr: Incomplete | None = ..., lstStyle: Incomplete | None = ..., p: Incomplete | None = ...
    ) -> None: ...

class Text(Serialisable):
    tagname: str
    strRef: Incomplete
    rich: Incomplete
    __elements__: Incomplete
    def __init__(self, strRef: Incomplete | None = ..., rich: Incomplete | None = ...) -> None: ...
    def to_tree(self, tagname: Incomplete | None = ..., idx: Incomplete | None = ..., namespace: Incomplete | None = ...): ...



from _typeshed import Incomplete

MAC_EPOCH: Incomplete
WINDOWS_EPOCH: Incomplete
# The following two constants are defined twice in the implementation.
CALENDAR_WINDOWS_1900 = WINDOWS_EPOCH  # noqa: F821
CALENDAR_MAC_1904 = MAC_EPOCH  # noqa: F821
SECS_PER_DAY: int
ISO_FORMAT: str
ISO_REGEX: Incomplete
ISO_DURATION: Incomplete

def to_ISO8601(dt): ...
def from_ISO8601(formatted_string): ...
def to_excel(dt, epoch=...): ...
def from_excel(value, epoch=..., timedelta: bool = ...): ...
def time_to_days(value): ...
def timedelta_to_days(value): ...
def days_to_time(value): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

from .cell_range import CellRange

class MergeCell(CellRange):
    tagname: str
    @property
    def ref(self): ...
    __attrs__: Incomplete
    def __init__(self, ref: Incomplete | None = ...) -> None: ...
    def __copy__(self): ...

class MergeCells(Serialisable):
    tagname: str
    # Overwritten by property below
    # count: Integer
    mergeCell: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(self, count: Incomplete | None = ..., mergeCell=...) -> None: ...
    @property
    def count(self): ...

class MergedCellRange(CellRange):
    ws: Incomplete
    start_cell: Incomplete
    def __init__(self, worksheet, coord) -> None: ...
    def format(self) -> None: ...
    def __contains__(self, coord): ...
    def __copy__(self): ...



from _typeshed import Incomplete
from abc import abstractmethod

from ._chart import ChartBase

class _LineChartBase(ChartBase):
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    dataLabels: Incomplete
    dropLines: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        grouping: str = ...,
        varyColors: Incomplete | None = ...,
        ser=...,
        dLbls: Incomplete | None = ...,
        dropLines: Incomplete | None = ...,
        **kw,
    ) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class LineChart(_LineChartBase):
    tagname: str
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    dropLines: Incomplete
    hiLowLines: Incomplete
    upDownBars: Incomplete
    marker: Incomplete
    smooth: Incomplete
    extLst: Incomplete
    x_axis: Incomplete
    y_axis: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        hiLowLines: Incomplete | None = ...,
        upDownBars: Incomplete | None = ...,
        marker: Incomplete | None = ...,
        smooth: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        **kw,
    ) -> None: ...

class LineChart3D(_LineChartBase):
    tagname: str
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    dropLines: Incomplete
    gapDepth: Incomplete
    hiLowLines: Incomplete
    upDownBars: Incomplete
    marker: Incomplete
    smooth: Incomplete
    extLst: Incomplete
    x_axis: Incomplete
    y_axis: Incomplete
    z_axis: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        gapDepth: Incomplete | None = ...,
        hiLowLines: Incomplete | None = ...,
        upDownBars: Incomplete | None = ...,
        marker: Incomplete | None = ...,
        smooth: Incomplete | None = ...,
        **kw,
    ) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.nested import NestedText
from openpyxl.descriptors.serialisable import Serialisable

class NumFmt(Serialisable):  # type: ignore[misc]
    formatCode: Incomplete
    sourceLinked: Incomplete
    def __init__(self, formatCode: Incomplete | None = ..., sourceLinked: bool = ...) -> None: ...

class NumberValueDescriptor(NestedText):
    allow_none: bool
    expected_type: Incomplete
    def __set__(self, instance, value) -> None: ...

class NumVal(Serialisable):  # type: ignore[misc]
    idx: Incomplete
    formatCode: Incomplete
    v: Incomplete
    def __init__(self, idx: Incomplete | None = ..., formatCode: Incomplete | None = ..., v: Incomplete | None = ...) -> None: ...

class NumData(Serialisable):  # type: ignore[misc]
    formatCode: Incomplete
    ptCount: Incomplete
    pt: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self, formatCode: Incomplete | None = ..., ptCount: Incomplete | None = ..., pt=..., extLst: Incomplete | None = ...
    ) -> None: ...

class NumRef(Serialisable):  # type: ignore[misc]
    f: Incomplete
    ref: Incomplete
    numCache: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self, f: Incomplete | None = ..., numCache: Incomplete | None = ..., extLst: Incomplete | None = ...
    ) -> None: ...

class StrVal(Serialisable):
    tagname: str
    idx: Incomplete
    v: Incomplete
    def __init__(self, idx: int = ..., v: Incomplete | None = ...) -> None: ...

class StrData(Serialisable):
    tagname: str
    ptCount: Incomplete
    pt: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(self, ptCount: Incomplete | None = ..., pt=..., extLst: Incomplete | None = ...) -> None: ...

class StrRef(Serialisable):
    tagname: str
    f: Incomplete
    strCache: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self, f: Incomplete | None = ..., strCache: Incomplete | None = ..., extLst: Incomplete | None = ...
    ) -> None: ...

class NumDataSource(Serialisable):  # type: ignore[misc]
    numRef: Incomplete
    numLit: Incomplete
    def __init__(self, numRef: Incomplete | None = ..., numLit: Incomplete | None = ...) -> None: ...

class Level(Serialisable):
    tagname: str
    pt: Incomplete
    __elements__: Incomplete
    def __init__(self, pt=...) -> None: ...

class MultiLevelStrData(Serialisable):
    tagname: str
    ptCount: Incomplete
    lvl: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(self, ptCount: Incomplete | None = ..., lvl=..., extLst: Incomplete | None = ...) -> None: ...

class MultiLevelStrRef(Serialisable):
    tagname: str
    f: Incomplete
    multiLvlStrCache: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self, f: Incomplete | None = ..., multiLvlStrCache: Incomplete | None = ..., extLst: Incomplete | None = ...
    ) -> None: ...

class AxDataSource(Serialisable):
    tagname: str
    numRef: Incomplete
    numLit: Incomplete
    strRef: Incomplete
    strLit: Incomplete
    multiLvlStrRef: Incomplete
    def __init__(
        self,
        numRef: Incomplete | None = ...,
        numLit: Incomplete | None = ...,
        strRef: Incomplete | None = ...,
        strLit: Incomplete | None = ...,
        multiLvlStrRef: Incomplete | None = ...,
    ) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class UpDownBars(Serialisable):
    tagname: str
    gapWidth: Incomplete
    upBars: Incomplete
    downBars: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        gapWidth: int = ...,
        upBars: Incomplete | None = ...,
        downBars: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class GraphicalProperties(Serialisable):
    tagname: str
    bwMode: Incomplete
    xfrm: Incomplete
    transform: Incomplete
    custGeom: Incomplete
    prstGeom: Incomplete
    noFill: Incomplete
    solidFill: Incomplete
    gradFill: Incomplete
    pattFill: Incomplete
    ln: Incomplete
    line: Incomplete
    scene3d: Incomplete
    sp3d: Incomplete
    shape3D: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        bwMode: Incomplete | None = ...,
        xfrm: Incomplete | None = ...,
        noFill: Incomplete | None = ...,
        solidFill: Incomplete | None = ...,
        gradFill: Incomplete | None = ...,
        pattFill: Incomplete | None = ...,
        ln: Incomplete | None = ...,
        scene3d: Incomplete | None = ...,
        custGeom: Incomplete | None = ...,
        prstGeom: Incomplete | None = ...,
        sp3d: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...



from _typeshed import Incomplete

class Comment:
    content: Incomplete
    author: Incomplete
    height: Incomplete
    width: Incomplete
    def __init__(self, text, author, height: int = ..., width: int = ...) -> None: ...
    @property
    def parent(self): ...
    def __eq__(self, other): ...
    def __copy__(self): ...
    def bind(self, cell) -> None: ...
    def unbind(self) -> None: ...
    @property
    def text(self): ...
    @text.setter
    def text(self, value) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class Font(Serialisable):
    UNDERLINE_DOUBLE: str
    UNDERLINE_DOUBLE_ACCOUNTING: str
    UNDERLINE_SINGLE: str
    UNDERLINE_SINGLE_ACCOUNTING: str
    name: Incomplete
    charset: Incomplete
    family: Incomplete
    sz: Incomplete
    size: Incomplete
    b: Incomplete
    bold: Incomplete
    i: Incomplete
    italic: Incomplete
    strike: Incomplete
    strikethrough: Incomplete
    outline: Incomplete
    shadow: Incomplete
    condense: Incomplete
    extend: Incomplete
    u: Incomplete
    underline: Incomplete
    vertAlign: Incomplete
    color: Incomplete
    scheme: Incomplete
    tagname: str
    __elements__: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        sz: Incomplete | None = ...,
        b: Incomplete | None = ...,
        i: Incomplete | None = ...,
        charset: Incomplete | None = ...,
        u: Incomplete | None = ...,
        strike: Incomplete | None = ...,
        color: Incomplete | None = ...,
        scheme: Incomplete | None = ...,
        family: Incomplete | None = ...,
        size: Incomplete | None = ...,
        bold: Incomplete | None = ...,
        italic: Incomplete | None = ...,
        strikethrough: Incomplete | None = ...,
        underline: Incomplete | None = ...,
        vertAlign: Incomplete | None = ...,
        outline: Incomplete | None = ...,
        shadow: Incomplete | None = ...,
        condense: Incomplete | None = ...,
        extend: Incomplete | None = ...,
    ) -> None: ...
    @classmethod
    def from_tree(cls, node): ...

DEFAULT_FONT: Incomplete



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class HierarchyUsage(Serialisable):
    tagname: str
    hierarchyUsage: Incomplete
    def __init__(self, hierarchyUsage: Incomplete | None = ...) -> None: ...

class ColHierarchiesUsage(Serialisable):
    tagname: str
    colHierarchyUsage: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(self, count: Incomplete | None = ..., colHierarchyUsage=...) -> None: ...
    @property
    def count(self): ...

class RowHierarchiesUsage(Serialisable):
    tagname: str
    rowHierarchyUsage: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(self, count: Incomplete | None = ..., rowHierarchyUsage=...) -> None: ...
    @property
    def count(self): ...

class PivotFilter(Serialisable):
    tagname: str
    fld: Incomplete
    mpFld: Incomplete
    type: Incomplete
    evalOrder: Incomplete
    id: Incomplete
    iMeasureHier: Incomplete
    iMeasureFld: Incomplete
    name: Incomplete
    description: Incomplete
    stringValue1: Incomplete
    stringValue2: Incomplete
    autoFilter: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        fld: Incomplete | None = ...,
        mpFld: Incomplete | None = ...,
        type: Incomplete | None = ...,
        evalOrder: Incomplete | None = ...,
        id: Incomplete | None = ...,
        iMeasureHier: Incomplete | None = ...,
        iMeasureFld: Incomplete | None = ...,
        name: Incomplete | None = ...,
        description: Incomplete | None = ...,
        stringValue1: Incomplete | None = ...,
        stringValue2: Incomplete | None = ...,
        autoFilter: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class PivotFilters(Serialisable):  # type: ignore[misc]
    count: Incomplete
    filter: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., filter: Incomplete | None = ...) -> None: ...

class PivotTableStyle(Serialisable):
    tagname: str
    name: Incomplete
    showRowHeaders: Incomplete
    showColHeaders: Incomplete
    showRowStripes: Incomplete
    showColStripes: Incomplete
    showLastColumn: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        showRowHeaders: Incomplete | None = ...,
        showColHeaders: Incomplete | None = ...,
        showRowStripes: Incomplete | None = ...,
        showColStripes: Incomplete | None = ...,
        showLastColumn: Incomplete | None = ...,
    ) -> None: ...

class MemberList(Serialisable):
    tagname: str
    level: Incomplete
    member: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., level: Incomplete | None = ..., member=...) -> None: ...
    @property
    def count(self): ...

class MemberProperty(Serialisable):
    tagname: str
    name: Incomplete
    showCell: Incomplete
    showTip: Incomplete
    showAsCaption: Incomplete
    nameLen: Incomplete
    pPos: Incomplete
    pLen: Incomplete
    level: Incomplete
    field: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        showCell: Incomplete | None = ...,
        showTip: Incomplete | None = ...,
        showAsCaption: Incomplete | None = ...,
        nameLen: Incomplete | None = ...,
        pPos: Incomplete | None = ...,
        pLen: Incomplete | None = ...,
        level: Incomplete | None = ...,
        field: Incomplete | None = ...,
    ) -> None: ...

class PivotHierarchy(Serialisable):
    tagname: str
    outline: Incomplete
    multipleItemSelectionAllowed: Incomplete
    subtotalTop: Incomplete
    showInFieldList: Incomplete
    dragToRow: Incomplete
    dragToCol: Incomplete
    dragToPage: Incomplete
    dragToData: Incomplete
    dragOff: Incomplete
    includeNewItemsInFilter: Incomplete
    caption: Incomplete
    mps: Incomplete
    members: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        outline: Incomplete | None = ...,
        multipleItemSelectionAllowed: Incomplete | None = ...,
        subtotalTop: Incomplete | None = ...,
        showInFieldList: Incomplete | None = ...,
        dragToRow: Incomplete | None = ...,
        dragToCol: Incomplete | None = ...,
        dragToPage: Incomplete | None = ...,
        dragToData: Incomplete | None = ...,
        dragOff: Incomplete | None = ...,
        includeNewItemsInFilter: Incomplete | None = ...,
        caption: Incomplete | None = ...,
        mps=...,
        members: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class Reference(Serialisable):
    tagname: str
    field: Incomplete
    selected: Incomplete
    byPosition: Incomplete
    relative: Incomplete
    defaultSubtotal: Incomplete
    sumSubtotal: Incomplete
    countASubtotal: Incomplete
    avgSubtotal: Incomplete
    maxSubtotal: Incomplete
    minSubtotal: Incomplete
    productSubtotal: Incomplete
    countSubtotal: Incomplete
    stdDevSubtotal: Incomplete
    stdDevPSubtotal: Incomplete
    varSubtotal: Incomplete
    varPSubtotal: Incomplete
    x: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        field: Incomplete | None = ...,
        count: Incomplete | None = ...,
        selected: Incomplete | None = ...,
        byPosition: Incomplete | None = ...,
        relative: Incomplete | None = ...,
        defaultSubtotal: Incomplete | None = ...,
        sumSubtotal: Incomplete | None = ...,
        countASubtotal: Incomplete | None = ...,
        avgSubtotal: Incomplete | None = ...,
        maxSubtotal: Incomplete | None = ...,
        minSubtotal: Incomplete | None = ...,
        productSubtotal: Incomplete | None = ...,
        countSubtotal: Incomplete | None = ...,
        stdDevSubtotal: Incomplete | None = ...,
        stdDevPSubtotal: Incomplete | None = ...,
        varSubtotal: Incomplete | None = ...,
        varPSubtotal: Incomplete | None = ...,
        x: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...
    @property
    def count(self): ...

class PivotArea(Serialisable):
    tagname: str
    references: Incomplete
    extLst: Incomplete
    field: Incomplete
    type: Incomplete
    dataOnly: Incomplete
    labelOnly: Incomplete
    grandRow: Incomplete
    grandCol: Incomplete
    cacheIndex: Incomplete
    outline: Incomplete
    offset: Incomplete
    collapsedLevelsAreSubtotals: Incomplete
    axis: Incomplete
    fieldPosition: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        references=...,
        extLst: Incomplete | None = ...,
        field: Incomplete | None = ...,
        type: str = ...,
        dataOnly: bool = ...,
        labelOnly: Incomplete | None = ...,
        grandRow: Incomplete | None = ...,
        grandCol: Incomplete | None = ...,
        cacheIndex: Incomplete | None = ...,
        outline: bool = ...,
        offset: Incomplete | None = ...,
        collapsedLevelsAreSubtotals: Incomplete | None = ...,
        axis: Incomplete | None = ...,
        fieldPosition: Incomplete | None = ...,
    ) -> None: ...

class ChartFormat(Serialisable):
    tagname: str
    chart: Incomplete
    format: Incomplete
    series: Incomplete
    pivotArea: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        chart: Incomplete | None = ...,
        format: Incomplete | None = ...,
        series: Incomplete | None = ...,
        pivotArea: Incomplete | None = ...,
    ) -> None: ...

class ConditionalFormat(Serialisable):
    tagname: str
    scope: Incomplete
    type: Incomplete
    priority: Incomplete
    pivotAreas: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        scope: Incomplete | None = ...,
        type: Incomplete | None = ...,
        priority: Incomplete | None = ...,
        pivotAreas=...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class Format(Serialisable):
    tagname: str
    action: Incomplete
    dxfId: Incomplete
    pivotArea: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        action: str = ...,
        dxfId: Incomplete | None = ...,
        pivotArea: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class DataField(Serialisable):
    tagname: str
    name: Incomplete
    fld: Incomplete
    subtotal: Incomplete
    showDataAs: Incomplete
    baseField: Incomplete
    baseItem: Incomplete
    numFmtId: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        fld: Incomplete | None = ...,
        subtotal: str = ...,
        showDataAs: str = ...,
        baseField: int = ...,
        baseItem: int = ...,
        numFmtId: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class PageField(Serialisable):
    tagname: str
    fld: Incomplete
    item: Incomplete
    hier: Incomplete
    name: Incomplete
    cap: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        fld: Incomplete | None = ...,
        item: Incomplete | None = ...,
        hier: Incomplete | None = ...,
        name: Incomplete | None = ...,
        cap: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class RowColItem(Serialisable):
    tagname: str
    t: Incomplete
    r: Incomplete
    i: Incomplete
    x: Incomplete
    __elements__: Incomplete
    def __init__(self, t: str = ..., r: int = ..., i: int = ..., x=...) -> None: ...

class RowColField(Serialisable):
    tagname: str
    x: Incomplete
    def __init__(self, x: Incomplete | None = ...) -> None: ...

class AutoSortScope(Serialisable):  # type: ignore[misc]
    pivotArea: Incomplete
    __elements__: Incomplete
    def __init__(self, pivotArea: Incomplete | None = ...) -> None: ...

class FieldItem(Serialisable):
    tagname: str
    n: Incomplete
    t: Incomplete
    h: Incomplete
    s: Incomplete
    sd: Incomplete
    f: Incomplete
    m: Incomplete
    c: Incomplete
    x: Incomplete
    d: Incomplete
    e: Incomplete
    def __init__(
        self,
        n: Incomplete | None = ...,
        t: str = ...,
        h: Incomplete | None = ...,
        s: Incomplete | None = ...,
        sd: bool = ...,
        f: Incomplete | None = ...,
        m: Incomplete | None = ...,
        c: Incomplete | None = ...,
        x: Incomplete | None = ...,
        d: Incomplete | None = ...,
        e: Incomplete | None = ...,
    ) -> None: ...

class PivotField(Serialisable):
    tagname: str
    items: Incomplete
    autoSortScope: Incomplete
    extLst: Incomplete
    name: Incomplete
    axis: Incomplete
    dataField: Incomplete
    subtotalCaption: Incomplete
    showDropDowns: Incomplete
    hiddenLevel: Incomplete
    uniqueMemberProperty: Incomplete
    compact: Incomplete
    allDrilled: Incomplete
    numFmtId: Incomplete
    outline: Incomplete
    subtotalTop: Incomplete
    dragToRow: Incomplete
    dragToCol: Incomplete
    multipleItemSelectionAllowed: Incomplete
    dragToPage: Incomplete
    dragToData: Incomplete
    dragOff: Incomplete
    showAll: Incomplete
    insertBlankRow: Incomplete
    serverField: Incomplete
    insertPageBreak: Incomplete
    autoShow: Incomplete
    topAutoShow: Incomplete
    hideNewItems: Incomplete
    measureFilter: Incomplete
    includeNewItemsInFilter: Incomplete
    itemPageCount: Incomplete
    sortType: Incomplete
    dataSourceSort: Incomplete
    nonAutoSortDefault: Incomplete
    rankBy: Incomplete
    defaultSubtotal: Incomplete
    sumSubtotal: Incomplete
    countASubtotal: Incomplete
    avgSubtotal: Incomplete
    maxSubtotal: Incomplete
    minSubtotal: Incomplete
    productSubtotal: Incomplete
    countSubtotal: Incomplete
    stdDevSubtotal: Incomplete
    stdDevPSubtotal: Incomplete
    varSubtotal: Incomplete
    varPSubtotal: Incomplete
    showPropCell: Incomplete
    showPropTip: Incomplete
    showPropAsCaption: Incomplete
    defaultAttributeDrillState: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        items=...,
        autoSortScope: Incomplete | None = ...,
        name: Incomplete | None = ...,
        axis: Incomplete | None = ...,
        dataField: Incomplete | None = ...,
        subtotalCaption: Incomplete | None = ...,
        showDropDowns: bool = ...,
        hiddenLevel: Incomplete | None = ...,
        uniqueMemberProperty: Incomplete | None = ...,
        compact: bool = ...,
        allDrilled: Incomplete | None = ...,
        numFmtId: Incomplete | None = ...,
        outline: bool = ...,
        subtotalTop: bool = ...,
        dragToRow: bool = ...,
        dragToCol: bool = ...,
        multipleItemSelectionAllowed: Incomplete | None = ...,
        dragToPage: bool = ...,
        dragToData: bool = ...,
        dragOff: bool = ...,
        showAll: bool = ...,
        insertBlankRow: Incomplete | None = ...,
        serverField: Incomplete | None = ...,
        insertPageBreak: Incomplete | None = ...,
        autoShow: Incomplete | None = ...,
        topAutoShow: bool = ...,
        hideNewItems: Incomplete | None = ...,
        measureFilter: Incomplete | None = ...,
        includeNewItemsInFilter: Incomplete | None = ...,
        itemPageCount: int = ...,
        sortType: str = ...,
        dataSourceSort: Incomplete | None = ...,
        nonAutoSortDefault: Incomplete | None = ...,
        rankBy: Incomplete | None = ...,
        defaultSubtotal: bool = ...,
        sumSubtotal: Incomplete | None = ...,
        countASubtotal: Incomplete | None = ...,
        avgSubtotal: Incomplete | None = ...,
        maxSubtotal: Incomplete | None = ...,
        minSubtotal: Incomplete | None = ...,
        productSubtotal: Incomplete | None = ...,
        countSubtotal: Incomplete | None = ...,
        stdDevSubtotal: Incomplete | None = ...,
        stdDevPSubtotal: Incomplete | None = ...,
        varSubtotal: Incomplete | None = ...,
        varPSubtotal: Incomplete | None = ...,
        showPropCell: Incomplete | None = ...,
        showPropTip: Incomplete | None = ...,
        showPropAsCaption: Incomplete | None = ...,
        defaultAttributeDrillState: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class Location(Serialisable):
    tagname: str
    ref: Incomplete
    firstHeaderRow: Incomplete
    firstDataRow: Incomplete
    firstDataCol: Incomplete
    rowPageCount: Incomplete
    colPageCount: Incomplete
    def __init__(
        self,
        ref: Incomplete | None = ...,
        firstHeaderRow: Incomplete | None = ...,
        firstDataRow: Incomplete | None = ...,
        firstDataCol: Incomplete | None = ...,
        rowPageCount: Incomplete | None = ...,
        colPageCount: Incomplete | None = ...,
    ) -> None: ...

class TableDefinition(Serialisable):
    mime_type: str
    rel_type: str
    tagname: str
    cache: Incomplete
    name: Incomplete
    cacheId: Incomplete
    dataOnRows: Incomplete
    dataPosition: Incomplete
    dataCaption: Incomplete
    grandTotalCaption: Incomplete
    errorCaption: Incomplete
    showError: Incomplete
    missingCaption: Incomplete
    showMissing: Incomplete
    pageStyle: Incomplete
    pivotTableStyle: Incomplete
    vacatedStyle: Incomplete
    tag: Incomplete
    updatedVersion: Incomplete
    minRefreshableVersion: Incomplete
    asteriskTotals: Incomplete
    showItems: Incomplete
    editData: Incomplete
    disableFieldList: Incomplete
    showCalcMbrs: Incomplete
    visualTotals: Incomplete
    showMultipleLabel: Incomplete
    showDataDropDown: Incomplete
    showDrill: Incomplete
    printDrill: Incomplete
    showMemberPropertyTips: Incomplete
    showDataTips: Incomplete
    enableWizard: Incomplete
    enableDrill: Incomplete
    enableFieldProperties: Incomplete
    preserveFormatting: Incomplete
    useAutoFormatting: Incomplete
    pageWrap: Incomplete
    pageOverThenDown: Incomplete
    subtotalHiddenItems: Incomplete
    rowGrandTotals: Incomplete
    colGrandTotals: Incomplete
    fieldPrintTitles: Incomplete
    itemPrintTitles: Incomplete
    mergeItem: Incomplete
    showDropZones: Incomplete
    createdVersion: Incomplete
    indent: Incomplete
    showEmptyRow: Incomplete
    showEmptyCol: Incomplete
    showHeaders: Incomplete
    compact: Incomplete
    outline: Incomplete
    outlineData: Incomplete
    compactData: Incomplete
    published: Incomplete
    gridDropZones: Incomplete
    immersive: Incomplete
    multipleFieldFilters: Incomplete
    chartFormat: Incomplete
    rowHeaderCaption: Incomplete
    colHeaderCaption: Incomplete
    fieldListSortAscending: Incomplete
    mdxSubqueries: Incomplete
    customListSort: Incomplete
    autoFormatId: Incomplete
    applyNumberFormats: Incomplete
    applyBorderFormats: Incomplete
    applyFontFormats: Incomplete
    applyPatternFormats: Incomplete
    applyAlignmentFormats: Incomplete
    applyWidthHeightFormats: Incomplete
    location: Incomplete
    pivotFields: Incomplete
    rowFields: Incomplete
    rowItems: Incomplete
    colFields: Incomplete
    colItems: Incomplete
    pageFields: Incomplete
    dataFields: Incomplete
    formats: Incomplete
    conditionalFormats: Incomplete
    chartFormats: Incomplete
    pivotHierarchies: Incomplete
    pivotTableStyleInfo: Incomplete
    filters: Incomplete
    rowHierarchiesUsage: Incomplete
    colHierarchiesUsage: Incomplete
    extLst: Incomplete
    id: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        cacheId: Incomplete | None = ...,
        dataOnRows: bool = ...,
        dataPosition: Incomplete | None = ...,
        dataCaption: Incomplete | None = ...,
        grandTotalCaption: Incomplete | None = ...,
        errorCaption: Incomplete | None = ...,
        showError: bool = ...,
        missingCaption: Incomplete | None = ...,
        showMissing: bool = ...,
        pageStyle: Incomplete | None = ...,
        pivotTableStyle: Incomplete | None = ...,
        vacatedStyle: Incomplete | None = ...,
        tag: Incomplete | None = ...,
        updatedVersion: int = ...,
        minRefreshableVersion: int = ...,
        asteriskTotals: bool = ...,
        showItems: bool = ...,
        editData: bool = ...,
        disableFieldList: bool = ...,
        showCalcMbrs: bool = ...,
        visualTotals: bool = ...,
        showMultipleLabel: bool = ...,
        showDataDropDown: bool = ...,
        showDrill: bool = ...,
        printDrill: bool = ...,
        showMemberPropertyTips: bool = ...,
        showDataTips: bool = ...,
        enableWizard: bool = ...,
        enableDrill: bool = ...,
        enableFieldProperties: bool = ...,
        preserveFormatting: bool = ...,
        useAutoFormatting: bool = ...,
        pageWrap: int = ...,
        pageOverThenDown: bool = ...,
        subtotalHiddenItems: bool = ...,
        rowGrandTotals: bool = ...,
        colGrandTotals: bool = ...,
        fieldPrintTitles: bool = ...,
        itemPrintTitles: bool = ...,
        mergeItem: bool = ...,
        showDropZones: bool = ...,
        createdVersion: int = ...,
        indent: int = ...,
        showEmptyRow: bool = ...,
        showEmptyCol: bool = ...,
        showHeaders: bool = ...,
        compact: bool = ...,
        outline: bool = ...,
        outlineData: bool = ...,
        compactData: bool = ...,
        published: bool = ...,
        gridDropZones: bool = ...,
        immersive: bool = ...,
        multipleFieldFilters: Incomplete | None = ...,
        chartFormat: int = ...,
        rowHeaderCaption: Incomplete | None = ...,
        colHeaderCaption: Incomplete | None = ...,
        fieldListSortAscending: Incomplete | None = ...,
        mdxSubqueries: Incomplete | None = ...,
        customListSort: Incomplete | None = ...,
        autoFormatId: Incomplete | None = ...,
        applyNumberFormats: bool = ...,
        applyBorderFormats: bool = ...,
        applyFontFormats: bool = ...,
        applyPatternFormats: bool = ...,
        applyAlignmentFormats: bool = ...,
        applyWidthHeightFormats: bool = ...,
        location: Incomplete | None = ...,
        pivotFields=...,
        rowFields=...,
        rowItems=...,
        colFields=...,
        colItems=...,
        pageFields=...,
        dataFields=...,
        formats=...,
        conditionalFormats=...,
        chartFormats=...,
        pivotHierarchies=...,
        pivotTableStyleInfo: Incomplete | None = ...,
        filters=...,
        rowHierarchiesUsage: Incomplete | None = ...,
        colHierarchiesUsage: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        id: Incomplete | None = ...,
    ) -> None: ...
    def to_tree(self): ...
    @property
    def path(self): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class WebPublishItem(Serialisable):
    tagname: str
    id: Incomplete
    divId: Incomplete
    sourceType: Incomplete
    sourceRef: Incomplete
    sourceObject: Incomplete
    destinationFile: Incomplete
    title: Incomplete
    autoRepublish: Incomplete
    def __init__(
        self,
        id: Incomplete | None = ...,
        divId: Incomplete | None = ...,
        sourceType: Incomplete | None = ...,
        sourceRef: Incomplete | None = ...,
        sourceObject: Incomplete | None = ...,
        destinationFile: Incomplete | None = ...,
        title: Incomplete | None = ...,
        autoRepublish: Incomplete | None = ...,
    ) -> None: ...

class WebPublishItems(Serialisable):
    tagname: str
    count: Incomplete
    webPublishItem: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., webPublishItem: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class InputCells(Serialisable):
    tagname: str
    r: Incomplete
    deleted: Incomplete
    undone: Incomplete
    val: Incomplete
    numFmtId: Incomplete
    def __init__(
        self,
        r: Incomplete | None = ...,
        deleted: bool = ...,
        undone: bool = ...,
        val: Incomplete | None = ...,
        numFmtId: Incomplete | None = ...,
    ) -> None: ...

class Scenario(Serialisable):
    tagname: str
    inputCells: Incomplete
    name: Incomplete
    locked: Incomplete
    hidden: Incomplete
    user: Incomplete
    comment: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(
        self,
        inputCells=...,
        name: Incomplete | None = ...,
        locked: bool = ...,
        hidden: bool = ...,
        count: Incomplete | None = ...,
        user: Incomplete | None = ...,
        comment: Incomplete | None = ...,
    ) -> None: ...
    @property
    def count(self): ...

class ScenarioList(Serialisable):
    tagname: str
    scenario: Incomplete
    current: Incomplete
    show: Incomplete
    sqref: Incomplete
    __elements__: Incomplete
    def __init__(
        self, scenario=..., current: Incomplete | None = ..., show: Incomplete | None = ..., sqref: Incomplete | None = ...
    ) -> None: ...
    def append(self, scenario) -> None: ...
    def __bool__(self) -> bool: ...



from _typeshed import Incomplete

class IndexedList(list[Incomplete]):
    clean: bool
    def __init__(self, iterable: Incomplete | None = ...) -> None: ...
    def __contains__(self, value): ...
    def index(self, value): ...
    def append(self, value) -> None: ...
    def add(self, value): ...



from _typeshed import Incomplete
from collections.abc import Generator

COORD_RE: Incomplete
COL_RANGE: str
ROW_RANGE: str
RANGE_EXPR: str
ABSOLUTE_RE: Incomplete
SHEET_TITLE: str
SHEETRANGE_RE: Incomplete

def get_column_interval(start: str | int, end: str | int) -> list[str]: ...
def coordinate_from_string(coord_string: str) -> tuple[str, int]: ...
def absolute_coordinate(coord_string: str) -> str: ...

col: Incomplete

def get_column_letter(idx: int) -> str: ...
def column_index_from_string(str_col: str) -> int: ...
def range_boundaries(range_string: str) -> tuple[int, int, int, int]: ...
def rows_from_range(range_string) -> Generator[Incomplete, None, None]: ...
def cols_from_range(range_string) -> Generator[Incomplete, None, None]: ...
def coordinate_to_tuple(coordinate: str) -> tuple[int, int]: ...
def range_to_tuple(range_string: str) -> tuple[str, tuple[int, int, int, int]]: ...
def quote_sheetname(sheetname: str) -> str: ...



from _typeshed import Incomplete
from collections.abc import Generator

from openpyxl.descriptors.serialisable import Serialisable

RESERVED: Incomplete
RESERVED_REGEX: Incomplete
COL_RANGE: str
COL_RANGE_RE: Incomplete
ROW_RANGE: str
ROW_RANGE_RE: Incomplete
TITLES_REGEX: Incomplete

class DefinedName(Serialisable):
    tagname: str
    name: Incomplete
    comment: Incomplete
    customMenu: Incomplete
    description: Incomplete
    help: Incomplete
    statusBar: Incomplete
    localSheetId: Incomplete
    hidden: Incomplete
    function: Incomplete
    vbProcedure: Incomplete
    xlm: Incomplete
    functionGroupId: Incomplete
    shortcutKey: Incomplete
    publishToServer: Incomplete
    workbookParameter: Incomplete
    attr_text: Incomplete
    value: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        comment: Incomplete | None = ...,
        customMenu: Incomplete | None = ...,
        description: Incomplete | None = ...,
        help: Incomplete | None = ...,
        statusBar: Incomplete | None = ...,
        localSheetId: Incomplete | None = ...,
        hidden: Incomplete | None = ...,
        function: Incomplete | None = ...,
        vbProcedure: Incomplete | None = ...,
        xlm: Incomplete | None = ...,
        functionGroupId: Incomplete | None = ...,
        shortcutKey: Incomplete | None = ...,
        publishToServer: Incomplete | None = ...,
        workbookParameter: Incomplete | None = ...,
        attr_text: Incomplete | None = ...,
    ) -> None: ...
    @property
    def type(self): ...
    @property
    def destinations(self) -> Generator[Incomplete, None, None]: ...
    @property
    def is_reserved(self): ...
    @property
    def is_external(self): ...
    def __iter__(self): ...

class DefinedNameList(Serialisable):
    tagname: str
    definedName: Incomplete
    def __init__(self, definedName=...) -> None: ...
    def append(self, defn) -> None: ...
    def __len__(self) -> int: ...
    def __contains__(self, name): ...
    def __getitem__(self, name): ...
    def get(self, name, scope: Incomplete | None = ...): ...
    def __delitem__(self, name) -> None: ...
    def delete(self, name, scope: Incomplete | None = ...): ...
    def localnames(self, scope): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class TableStyleElement(Serialisable):
    tagname: str
    type: Incomplete
    size: Incomplete
    dxfId: Incomplete
    def __init__(self, type: Incomplete | None = ..., size: Incomplete | None = ..., dxfId: Incomplete | None = ...) -> None: ...

class TableStyle(Serialisable):
    tagname: str
    name: Incomplete
    pivot: Incomplete
    table: Incomplete
    count: Incomplete
    tableStyleElement: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        pivot: Incomplete | None = ...,
        table: Incomplete | None = ...,
        count: Incomplete | None = ...,
        tableStyleElement=...,
    ) -> None: ...

class TableStyleList(Serialisable):
    tagname: str
    defaultTableStyle: Incomplete
    defaultPivotStyle: Incomplete
    tableStyle: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(
        self, count: Incomplete | None = ..., defaultTableStyle: str = ..., defaultPivotStyle: str = ..., tableStyle=...
    ) -> None: ...
    @property
    def count(self): ...



from _typeshed import Incomplete

from openpyxl.descriptors import Strict
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.styles.styleable import StyleableObject
from openpyxl.utils.bound_dictionary import BoundDictionary

class Dimension(Strict, StyleableObject):
    __fields__: Incomplete
    index: Incomplete
    hidden: Incomplete
    outlineLevel: Incomplete
    outline_level: Incomplete
    collapsed: Incomplete
    style: Incomplete
    def __init__(
        self, index, hidden, outlineLevel, collapsed, worksheet, visible: bool = ..., style: Incomplete | None = ...
    ) -> None: ...
    def __iter__(self): ...
    def __copy__(self): ...

class RowDimension(Dimension):
    __fields__: Incomplete
    r: Incomplete
    s: Incomplete
    ht: Incomplete
    height: Incomplete
    thickBot: Incomplete
    thickTop: Incomplete
    def __init__(
        self,
        worksheet,
        index: int = ...,
        ht: Incomplete | None = ...,
        customHeight: Incomplete | None = ...,
        s: Incomplete | None = ...,
        customFormat: Incomplete | None = ...,
        hidden: bool = ...,
        outlineLevel: int = ...,
        outline_level: Incomplete | None = ...,
        collapsed: bool = ...,
        visible: Incomplete | None = ...,
        height: Incomplete | None = ...,
        r: Incomplete | None = ...,
        spans: Incomplete | None = ...,
        thickBot: Incomplete | None = ...,
        thickTop: Incomplete | None = ...,
        **kw,
    ) -> None: ...
    @property
    def customFormat(self): ...
    @property
    def customHeight(self): ...

class ColumnDimension(Dimension):
    width: Incomplete
    bestFit: Incomplete
    auto_size: Incomplete
    index: Incomplete
    min: Incomplete
    max: Incomplete
    collapsed: Incomplete
    __fields__: Incomplete
    def __init__(
        self,
        worksheet,
        index: str = ...,
        width=...,
        bestFit: bool = ...,
        hidden: bool = ...,
        outlineLevel: int = ...,
        outline_level: Incomplete | None = ...,
        collapsed: bool = ...,
        style: Incomplete | None = ...,
        min: Incomplete | None = ...,
        max: Incomplete | None = ...,
        customWidth: bool = ...,
        visible: Incomplete | None = ...,
        auto_size: Incomplete | None = ...,
    ) -> None: ...
    @property
    def customWidth(self): ...
    def reindex(self) -> None: ...
    def to_tree(self): ...

class DimensionHolder(BoundDictionary):
    worksheet: Incomplete
    max_outline: Incomplete
    default_factory: Incomplete
    def __init__(self, worksheet, reference: str = ..., default_factory: Incomplete | None = ...) -> None: ...
    def group(self, start, end: Incomplete | None = ..., outline_level: int = ..., hidden: bool = ...) -> None: ...
    def to_tree(self): ...

class SheetFormatProperties(Serialisable):
    tagname: str
    baseColWidth: Incomplete
    defaultColWidth: Incomplete
    defaultRowHeight: Incomplete
    customHeight: Incomplete
    zeroHeight: Incomplete
    thickTop: Incomplete
    thickBottom: Incomplete
    outlineLevelRow: Incomplete
    outlineLevelCol: Incomplete
    def __init__(
        self,
        baseColWidth: int = ...,
        defaultColWidth: Incomplete | None = ...,
        defaultRowHeight: int = ...,
        customHeight: Incomplete | None = ...,
        zeroHeight: Incomplete | None = ...,
        thickTop: Incomplete | None = ...,
        thickBottom: Incomplete | None = ...,
        outlineLevelRow: Incomplete | None = ...,
        outlineLevelCol: Incomplete | None = ...,
    ) -> None: ...

class SheetDimension(Serialisable):
    tagname: str
    ref: Incomplete
    def __init__(self, ref: Incomplete | None = ...) -> None: ...
    @property
    def boundaries(self): ...



from _typeshed import Incomplete

MIN_ROW: int
MIN_COLUMN: int
MAX_COLUMN: int
MAX_ROW: int
PACKAGE_PROPS: str
PACKAGE_XL: str
PACKAGE_RELS: str
PACKAGE_THEME: Incomplete
PACKAGE_WORKSHEETS: Incomplete
PACKAGE_CHARTSHEETS: Incomplete
PACKAGE_DRAWINGS: Incomplete
PACKAGE_CHARTS: Incomplete
PACKAGE_IMAGES: Incomplete
PACKAGE_WORKSHEET_RELS: Incomplete
PACKAGE_CHARTSHEETS_RELS: Incomplete
PACKAGE_PIVOT_TABLE: Incomplete
PACKAGE_PIVOT_CACHE: Incomplete
ARC_CONTENT_TYPES: str
ARC_ROOT_RELS: Incomplete
ARC_WORKBOOK_RELS: Incomplete
ARC_CORE: Incomplete
ARC_APP: Incomplete
ARC_WORKBOOK: Incomplete
ARC_STYLE: Incomplete
ARC_THEME: Incomplete
ARC_SHARED_STRINGS: Incomplete
ARC_CUSTOM_UI: str
XML_NS: str
DCORE_NS: str
DCTERMS_NS: str
DCTERMS_PREFIX: str
DOC_NS: str
REL_NS: Incomplete
COMMENTS_NS: Incomplete
IMAGE_NS: Incomplete
VML_NS: Incomplete
VTYPES_NS: Incomplete
XPROPS_NS: Incomplete
EXTERNAL_LINK_NS: Incomplete
PKG_NS: str
PKG_REL_NS: Incomplete
COREPROPS_NS: Incomplete
CONTYPES_NS: Incomplete
XSI_NS: str
SHEET_MAIN_NS: str
CHART_NS: str
DRAWING_NS: str
SHEET_DRAWING_NS: str
CHART_DRAWING_NS: str
CUSTOMUI_NS: str
NAMESPACES: Incomplete
WORKBOOK_MACRO: str
WORKBOOK: str
SPREADSHEET: str
SHARED_STRINGS: Incomplete
EXTERNAL_LINK: Incomplete
WORKSHEET_TYPE: Incomplete
COMMENTS_TYPE: Incomplete
STYLES_TYPE: Incomplete
CHARTSHEET_TYPE: Incomplete
DRAWING_TYPE: str
CHART_TYPE: str
CHARTSHAPE_TYPE: str
THEME_TYPE: str
XLTM: Incomplete
XLSM: Incomplete
XLTX: Incomplete
XLSX: Incomplete
EXT_TYPES: Incomplete
CTRL: str
ACTIVEX: str
VBA: str



from _typeshed import Incomplete
from abc import abstractmethod

from ._chart import ChartBase

class _AreaChartBase(ChartBase):
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    dataLabels: Incomplete
    dropLines: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        grouping: str = ...,
        varyColors: Incomplete | None = ...,
        ser=...,
        dLbls: Incomplete | None = ...,
        dropLines: Incomplete | None = ...,
    ) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class AreaChart(_AreaChartBase):
    tagname: str
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    dropLines: Incomplete
    x_axis: Incomplete
    y_axis: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(self, axId: Incomplete | None = ..., extLst: Incomplete | None = ..., **kw) -> None: ...

class AreaChart3D(AreaChart):
    tagname: str
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    dropLines: Incomplete
    gapDepth: Incomplete
    x_axis: Incomplete
    y_axis: Incomplete
    z_axis: Incomplete
    __elements__: Incomplete
    def __init__(self, gapDepth: Incomplete | None = ..., **kw) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class ConditionalFormatting(Serialisable):
    tagname: str
    sqref: Incomplete
    cells: Incomplete
    pivot: Incomplete
    cfRule: Incomplete
    rules: Incomplete
    def __init__(self, sqref=..., pivot: Incomplete | None = ..., cfRule=..., extLst: Incomplete | None = ...) -> None: ...
    def __eq__(self, other): ...
    def __hash__(self) -> int: ...
    def __contains__(self, coord): ...

class ConditionalFormattingList:
    max_priority: int
    def __init__(self) -> None: ...
    def add(self, range_string, cfRule) -> None: ...
    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self): ...
    def __getitem__(self, key): ...
    def __delitem__(self, key) -> None: ...
    def __setitem__(self, key, rule) -> None: ...



from _typeshed import Incomplete

class Drawing:
    count: int
    name: str
    description: str
    coordinates: Incomplete
    left: int
    top: int
    resize_proportional: bool
    rotation: int
    anchortype: str
    anchorcol: int
    anchorrow: int
    def __init__(self) -> None: ...
    @property
    def width(self): ...
    @width.setter
    def width(self, w) -> None: ...
    @property
    def height(self): ...
    @height.setter
    def height(self, h) -> None: ...
    def set_dimension(self, w: int = ..., h: int = ...) -> None: ...
    def get_emu_dimensions(self): ...
    @property
    def anchor(self): ...



from _typeshed import Incomplete
from array import array

from openpyxl.descriptors.serialisable import Serialisable

class ArrayDescriptor:
    key: Incomplete
    def __init__(self, key) -> None: ...
    def __get__(self, instance, cls): ...
    def __set__(self, instance, value) -> None: ...

class StyleArray(array[Incomplete]):
    tagname: str
    fontId: Incomplete
    fillId: Incomplete
    borderId: Incomplete
    numFmtId: Incomplete
    protectionId: Incomplete
    alignmentId: Incomplete
    pivotButton: Incomplete
    quotePrefix: Incomplete
    xfId: Incomplete
    def __new__(cls, args=...): ...
    def __hash__(self) -> int: ...
    def __copy__(self): ...
    def __deepcopy__(self, memo): ...

class CellStyle(Serialisable):
    tagname: str
    numFmtId: Incomplete
    fontId: Incomplete
    fillId: Incomplete
    borderId: Incomplete
    xfId: Incomplete
    quotePrefix: Incomplete
    pivotButton: Incomplete
    applyNumberFormat: Incomplete
    applyFont: Incomplete
    applyFill: Incomplete
    applyBorder: Incomplete
    # Overwritten by properties below
    # applyAlignment: Bool
    # applyProtection: Bool
    alignment: Incomplete
    protection: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(
        self,
        numFmtId: int = ...,
        fontId: int = ...,
        fillId: int = ...,
        borderId: int = ...,
        xfId: Incomplete | None = ...,
        quotePrefix: Incomplete | None = ...,
        pivotButton: Incomplete | None = ...,
        applyNumberFormat: Incomplete | None = ...,
        applyFont: Incomplete | None = ...,
        applyFill: Incomplete | None = ...,
        applyBorder: Incomplete | None = ...,
        applyAlignment: Incomplete | None = ...,
        applyProtection: Incomplete | None = ...,
        alignment: Incomplete | None = ...,
        protection: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...
    def to_array(self): ...
    @classmethod
    def from_array(cls, style): ...
    @property
    def applyProtection(self): ...
    @property
    def applyAlignment(self): ...

class CellStyleList(Serialisable):
    tagname: str
    __attrs__: Incomplete
    # Overwritten by property below
    # count: Integer
    xf: Incomplete
    alignment: Incomplete
    protection: Incomplete
    __elements__: Incomplete
    def __init__(self, count: Incomplete | None = ..., xf=...) -> None: ...
    @property
    def count(self): ...
    def __getitem__(self, idx): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class Hyperlink(Serialisable):
    tagname: str
    ref: Incomplete
    location: Incomplete
    tooltip: Incomplete
    display: Incomplete
    id: Incomplete
    target: Incomplete
    __attrs__: Incomplete
    def __init__(
        self,
        ref: Incomplete | None = ...,
        location: Incomplete | None = ...,
        tooltip: Incomplete | None = ...,
        display: Incomplete | None = ...,
        id: Incomplete | None = ...,
        target: Incomplete | None = ...,
    ) -> None: ...

class HyperlinkList(Serialisable):
    tagname: str
    hyperlink: Incomplete
    def __init__(self, hyperlink=...) -> None: ...
    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...
    def append(self, value) -> None: ...



from _typeshed import Incomplete

def read_dimension(source): ...

class ReadOnlyWorksheet:
    cell: Incomplete
    iter_rows: Incomplete
    @property
    def values(self): ...
    @property
    def rows(self): ...
    __getitem__: Incomplete
    __iter__: Incomplete
    parent: Incomplete
    title: Incomplete
    sheet_state: str
    def __init__(self, parent_workbook, title, worksheet_path, shared_strings) -> None: ...
    def calculate_dimension(self, force: bool = ...): ...
    def reset_dimensions(self) -> None: ...
    @property
    def min_row(self): ...
    @property
    def max_row(self): ...
    @property
    def min_column(self): ...
    @property
    def max_column(self): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class LineEndProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    type: Incomplete
    w: Incomplete
    len: Incomplete
    def __init__(self, type: Incomplete | None = ..., w: Incomplete | None = ..., len: Incomplete | None = ...) -> None: ...

class DashStop(Serialisable):
    tagname: str
    namespace: Incomplete
    d: Incomplete
    length: Incomplete
    sp: Incomplete
    space: Incomplete
    def __init__(self, d: int = ..., sp: int = ...) -> None: ...

class DashStopList(Serialisable):
    ds: Incomplete
    def __init__(self, ds: Incomplete | None = ...) -> None: ...

class LineProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    w: Incomplete
    width: Incomplete
    cap: Incomplete
    cmpd: Incomplete
    algn: Incomplete
    noFill: Incomplete
    solidFill: Incomplete
    gradFill: Incomplete
    pattFill: Incomplete
    prstDash: Incomplete
    dashStyle: Incomplete
    custDash: Incomplete
    round: Incomplete
    bevel: Incomplete
    miter: Incomplete
    headEnd: Incomplete
    tailEnd: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        w: Incomplete | None = ...,
        cap: Incomplete | None = ...,
        cmpd: Incomplete | None = ...,
        algn: Incomplete | None = ...,
        noFill: Incomplete | None = ...,
        solidFill: Incomplete | None = ...,
        gradFill: Incomplete | None = ...,
        pattFill: Incomplete | None = ...,
        prstDash: Incomplete | None = ...,
        custDash: Incomplete | None = ...,
        round: Incomplete | None = ...,
        bevel: Incomplete | None = ...,
        miter: Incomplete | None = ...,
        headEnd: Incomplete | None = ...,
        tailEnd: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors import Typed
from openpyxl.descriptors.serialisable import Serialisable

class Title(Serialisable):
    tagname: str
    tx: Incomplete
    text: Incomplete
    layout: Incomplete
    overlay: Incomplete
    spPr: Incomplete
    graphicalProperties: Incomplete
    txPr: Incomplete
    body: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        tx: Incomplete | None = ...,
        layout: Incomplete | None = ...,
        overlay: Incomplete | None = ...,
        spPr: Incomplete | None = ...,
        txPr: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

def title_maker(text): ...

class TitleDescriptor(Typed):
    expected_type: Incomplete
    allow_none: bool
    def __set__(self, instance, value) -> None: ...



from _typeshed import Incomplete

from . import _Serialiasable

KEYWORDS: Incomplete
seq_types: Incomplete

class Serialisable(_Serialiasable):
    __attrs__: Incomplete
    __nested__: Incomplete
    __elements__: Incomplete
    __namespaced__: Incomplete
    idx_base: int
    @property
    # TODO: needs overrides in many sub-classes
    # @abstractmethod
    def tagname(self) -> str: ...
    namespace: Incomplete
    @classmethod
    def from_tree(cls, node): ...
    def to_tree(self, tagname: Incomplete | None = ..., idx: Incomplete | None = ..., namespace: Incomplete | None = ...): ...
    def __iter__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __hash__(self) -> int: ...
    def __add__(self, other): ...
    def __copy__(self): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class Record(Serialisable):
    tagname: str
    m: Incomplete
    n: Incomplete
    b: Incomplete
    e: Incomplete
    s: Incomplete
    d: Incomplete
    x: Incomplete
    def __init__(
        self,
        _fields=...,
        m: Incomplete | None = ...,
        n: Incomplete | None = ...,
        b: Incomplete | None = ...,
        e: Incomplete | None = ...,
        s: Incomplete | None = ...,
        d: Incomplete | None = ...,
        x: Incomplete | None = ...,
    ) -> None: ...

class RecordList(Serialisable):
    mime_type: str
    rel_type: str
    tagname: str
    r: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(self, count: Incomplete | None = ..., r=..., extLst: Incomplete | None = ...) -> None: ...
    @property
    def count(self): ...
    def to_tree(self): ...
    @property
    def path(self): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class GroupShapeProperties(Serialisable):
    tagname: str
    bwMode: Incomplete
    xfrm: Incomplete
    scene3d: Incomplete
    extLst: Incomplete
    def __init__(
        self,
        bwMode: Incomplete | None = ...,
        xfrm: Incomplete | None = ...,
        scene3d: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class GroupLocking(Serialisable):
    tagname: str
    namespace: Incomplete
    noGrp: Incomplete
    noUngrp: Incomplete
    noSelect: Incomplete
    noRot: Incomplete
    noChangeAspect: Incomplete
    noMove: Incomplete
    noResize: Incomplete
    noChangeArrowheads: Incomplete
    noEditPoints: Incomplete
    noAdjustHandles: Incomplete
    noChangeShapeType: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        noGrp: Incomplete | None = ...,
        noUngrp: Incomplete | None = ...,
        noSelect: Incomplete | None = ...,
        noRot: Incomplete | None = ...,
        noChangeAspect: Incomplete | None = ...,
        noChangeArrowheads: Incomplete | None = ...,
        noMove: Incomplete | None = ...,
        noResize: Incomplete | None = ...,
        noEditPoints: Incomplete | None = ...,
        noAdjustHandles: Incomplete | None = ...,
        noChangeShapeType: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class NonVisualGroupDrawingShapeProps(Serialisable):
    tagname: str
    grpSpLocks: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(self, grpSpLocks: Incomplete | None = ..., extLst: Incomplete | None = ...) -> None: ...

class NonVisualDrawingShapeProps(Serialisable):
    tagname: str
    spLocks: Incomplete
    txBax: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    txBox: Incomplete
    def __init__(
        self, spLocks: Incomplete | None = ..., txBox: Incomplete | None = ..., extLst: Incomplete | None = ...
    ) -> None: ...

class NonVisualDrawingProps(Serialisable):
    tagname: str
    id: Incomplete
    name: Incomplete
    descr: Incomplete
    hidden: Incomplete
    title: Incomplete
    hlinkClick: Incomplete
    hlinkHover: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        id: Incomplete | None = ...,
        name: Incomplete | None = ...,
        descr: Incomplete | None = ...,
        hidden: Incomplete | None = ...,
        title: Incomplete | None = ...,
        hlinkClick: Incomplete | None = ...,
        hlinkHover: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class NonVisualGroupShape(Serialisable):
    tagname: str
    cNvPr: Incomplete
    cNvGrpSpPr: Incomplete
    __elements__: Incomplete
    def __init__(self, cNvPr: Incomplete | None = ..., cNvGrpSpPr: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class DataTable(Serialisable):
    tagname: str
    showHorzBorder: Incomplete
    showVertBorder: Incomplete
    showOutline: Incomplete
    showKeys: Incomplete
    spPr: Incomplete
    graphicalProperties: Incomplete
    txPr: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        showHorzBorder: Incomplete | None = ...,
        showVertBorder: Incomplete | None = ...,
        showOutline: Incomplete | None = ...,
        showKeys: Incomplete | None = ...,
        spPr: Incomplete | None = ...,
        txPr: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class PlotArea(Serialisable):
    tagname: str
    layout: Incomplete
    dTable: Incomplete
    spPr: Incomplete
    graphicalProperties: Incomplete
    extLst: Incomplete
    areaChart: Incomplete
    area3DChart: Incomplete
    lineChart: Incomplete
    line3DChart: Incomplete
    stockChart: Incomplete
    radarChart: Incomplete
    scatterChart: Incomplete
    pieChart: Incomplete
    pie3DChart: Incomplete
    doughnutChart: Incomplete
    barChart: Incomplete
    bar3DChart: Incomplete
    ofPieChart: Incomplete
    surfaceChart: Incomplete
    surface3DChart: Incomplete
    bubbleChart: Incomplete
    valAx: Incomplete
    catAx: Incomplete
    dateAx: Incomplete
    serAx: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        layout: Incomplete | None = ...,
        dTable: Incomplete | None = ...,
        spPr: Incomplete | None = ...,
        _charts=...,
        _axes=...,
        extLst: Incomplete | None = ...,
    ) -> None: ...
    def to_tree(self, tagname: Incomplete | None = ..., idx: Incomplete | None = ..., namespace: Incomplete | None = ...): ...
    @classmethod
    def from_tree(cls, node): ...



from _typeshed import Incomplete
from collections.abc import Generator

def dataframe_to_rows(df, index: bool = ..., header: bool = ...) -> Generator[Incomplete, None, None]: ...
def expand_index(index, header: bool = ...) -> Generator[Incomplete, None, None]: ...



from _typeshed import Incomplete
from abc import abstractmethod

from openpyxl.descriptors.serialisable import Serialisable

from ._3d import _3DBase
from ._chart import ChartBase

class BandFormat(Serialisable):
    tagname: str
    idx: Incomplete
    spPr: Incomplete
    graphicalProperties: Incomplete
    __elements__: Incomplete
    def __init__(self, idx: int = ..., spPr: Incomplete | None = ...) -> None: ...

class BandFormatList(Serialisable):
    tagname: str
    bandFmt: Incomplete
    __elements__: Incomplete
    def __init__(self, bandFmt=...) -> None: ...

class _SurfaceChartBase(ChartBase):
    wireframe: Incomplete
    ser: Incomplete
    bandFmts: Incomplete
    __elements__: Incomplete
    def __init__(self, wireframe: Incomplete | None = ..., ser=..., bandFmts: Incomplete | None = ..., **kw) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class SurfaceChart3D(_SurfaceChartBase, _3DBase):
    tagname: str
    wireframe: Incomplete
    ser: Incomplete
    bandFmts: Incomplete
    extLst: Incomplete
    x_axis: Incomplete
    y_axis: Incomplete
    z_axis: Incomplete
    __elements__: Incomplete
    def __init__(self, **kw) -> None: ...

class SurfaceChart(SurfaceChart3D):
    tagname: str
    wireframe: Incomplete
    ser: Incomplete
    bandFmts: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(self, **kw) -> None: ...



from _typeshed import Incomplete
from collections import defaultdict

class BoundDictionary(defaultdict[Incomplete, Incomplete]):
    reference: Incomplete
    def __init__(self, reference: Incomplete | None = ..., *args, **kw) -> None: ...
    def __getitem__(self, key): ...



from _typeshed import Incomplete
from collections.abc import Generator

from openpyxl.descriptors.serialisable import Serialisable

class Properties(Serialisable):
    locked: Incomplete
    defaultSize: Incomplete
    disabled: Incomplete
    uiObject: Incomplete
    autoFill: Incomplete
    autoLine: Incomplete
    altText: Incomplete
    textHAlign: Incomplete
    textVAlign: Incomplete
    lockText: Incomplete
    justLastX: Incomplete
    autoScale: Incomplete
    rowHidden: Incomplete
    colHidden: Incomplete
    __elements__: Incomplete
    anchor: Incomplete
    def __init__(
        self,
        locked: Incomplete | None = ...,
        defaultSize: Incomplete | None = ...,
        _print: Incomplete | None = ...,
        disabled: Incomplete | None = ...,
        uiObject: Incomplete | None = ...,
        autoFill: Incomplete | None = ...,
        autoLine: Incomplete | None = ...,
        altText: Incomplete | None = ...,
        textHAlign: Incomplete | None = ...,
        textVAlign: Incomplete | None = ...,
        lockText: Incomplete | None = ...,
        justLastX: Incomplete | None = ...,
        autoScale: Incomplete | None = ...,
        rowHidden: Incomplete | None = ...,
        colHidden: Incomplete | None = ...,
        anchor: Incomplete | None = ...,
    ) -> None: ...

class CommentRecord(Serialisable):
    tagname: str
    ref: Incomplete
    authorId: Incomplete
    guid: Incomplete
    shapeId: Incomplete
    text: Incomplete
    commentPr: Incomplete
    author: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    height: Incomplete
    width: Incomplete
    def __init__(
        self,
        ref: str = ...,
        authorId: int = ...,
        guid: Incomplete | None = ...,
        shapeId: int = ...,
        text: Incomplete | None = ...,
        commentPr: Incomplete | None = ...,
        author: Incomplete | None = ...,
        height: int = ...,
        width: int = ...,
    ) -> None: ...
    @classmethod
    def from_cell(cls, cell): ...
    @property
    def content(self): ...

class CommentSheet(Serialisable):
    tagname: str
    authors: Incomplete
    commentList: Incomplete
    extLst: Incomplete
    mime_type: str
    __elements__: Incomplete
    def __init__(
        self, authors: Incomplete | None = ..., commentList: Incomplete | None = ..., extLst: Incomplete | None = ...
    ) -> None: ...
    def to_tree(self): ...
    @property
    def comments(self) -> Generator[Incomplete, None, None]: ...
    @classmethod
    def from_comments(cls, comments): ...
    def write_shapes(self, vml: Incomplete | None = ...): ...
    @property
    def path(self): ...



from _typeshed import Incomplete

from openpyxl.descriptors import Float
from openpyxl.descriptors.serialisable import Serialisable

class ValueDescriptor(Float):
    expected_type: Incomplete
    def __set__(self, instance, value) -> None: ...

class FormatObject(Serialisable):
    tagname: str
    type: Incomplete
    val: Incomplete
    gte: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self, type, val: Incomplete | None = ..., gte: Incomplete | None = ..., extLst: Incomplete | None = ...
    ) -> None: ...

class RuleType(Serialisable):  # type: ignore[misc]
    cfvo: Incomplete

class IconSet(RuleType):
    tagname: str
    iconSet: Incomplete
    showValue: Incomplete
    percent: Incomplete
    reverse: Incomplete
    __elements__: Incomplete
    cfvo: Incomplete
    def __init__(
        self,
        iconSet: Incomplete | None = ...,
        showValue: Incomplete | None = ...,
        percent: Incomplete | None = ...,
        reverse: Incomplete | None = ...,
        cfvo: Incomplete | None = ...,
    ) -> None: ...

class DataBar(RuleType):
    tagname: str
    minLength: Incomplete
    maxLength: Incomplete
    showValue: Incomplete
    color: Incomplete
    __elements__: Incomplete
    cfvo: Incomplete
    def __init__(
        self,
        minLength: Incomplete | None = ...,
        maxLength: Incomplete | None = ...,
        showValue: Incomplete | None = ...,
        cfvo: Incomplete | None = ...,
        color: Incomplete | None = ...,
    ) -> None: ...

class ColorScale(RuleType):
    tagname: str
    color: Incomplete
    __elements__: Incomplete
    cfvo: Incomplete
    def __init__(self, cfvo: Incomplete | None = ..., color: Incomplete | None = ...) -> None: ...

class Rule(Serialisable):
    tagname: str
    type: Incomplete
    dxfId: Incomplete
    priority: Incomplete
    stopIfTrue: Incomplete
    aboveAverage: Incomplete
    percent: Incomplete
    bottom: Incomplete
    operator: Incomplete
    text: Incomplete
    timePeriod: Incomplete
    rank: Incomplete
    stdDev: Incomplete
    equalAverage: Incomplete
    formula: Incomplete
    colorScale: Incomplete
    dataBar: Incomplete
    iconSet: Incomplete
    extLst: Incomplete
    dxf: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(
        self,
        type,
        dxfId: Incomplete | None = ...,
        priority: int = ...,
        stopIfTrue: Incomplete | None = ...,
        aboveAverage: Incomplete | None = ...,
        percent: Incomplete | None = ...,
        bottom: Incomplete | None = ...,
        operator: Incomplete | None = ...,
        text: Incomplete | None = ...,
        timePeriod: Incomplete | None = ...,
        rank: Incomplete | None = ...,
        stdDev: Incomplete | None = ...,
        equalAverage: Incomplete | None = ...,
        formula=...,
        colorScale: Incomplete | None = ...,
        dataBar: Incomplete | None = ...,
        iconSet: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        dxf: Incomplete | None = ...,
    ) -> None: ...

def ColorScaleRule(
    start_type: Incomplete | None = ...,
    start_value: Incomplete | None = ...,
    start_color: Incomplete | None = ...,
    mid_type: Incomplete | None = ...,
    mid_value: Incomplete | None = ...,
    mid_color: Incomplete | None = ...,
    end_type: Incomplete | None = ...,
    end_value: Incomplete | None = ...,
    end_color: Incomplete | None = ...,
): ...
def FormulaRule(
    formula: Incomplete | None = ...,
    stopIfTrue: Incomplete | None = ...,
    font: Incomplete | None = ...,
    border: Incomplete | None = ...,
    fill: Incomplete | None = ...,
): ...
def CellIsRule(
    operator: Incomplete | None = ...,
    formula: Incomplete | None = ...,
    stopIfTrue: Incomplete | None = ...,
    font: Incomplete | None = ...,
    border: Incomplete | None = ...,
    fill: Incomplete | None = ...,
): ...
def IconSetRule(
    icon_style: Incomplete | None = ...,
    type: Incomplete | None = ...,
    values: Incomplete | None = ...,
    showValue: Incomplete | None = ...,
    percent: Incomplete | None = ...,
    reverse: Incomplete | None = ...,
): ...
def DataBarRule(
    start_type: Incomplete | None = ...,
    start_value: Incomplete | None = ...,
    end_type: Incomplete | None = ...,
    end_value: Incomplete | None = ...,
    color: Incomplete | None = ...,
    showValue: Incomplete | None = ...,
    minLength: Incomplete | None = ...,
    maxLength: Incomplete | None = ...,
): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class CustomChartsheetView(Serialisable):
    tagname: str
    guid: Incomplete
    scale: Incomplete
    state: Incomplete
    zoomToFit: Incomplete
    pageMargins: Incomplete
    pageSetup: Incomplete
    headerFooter: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        guid: Incomplete | None = ...,
        scale: Incomplete | None = ...,
        state: str = ...,
        zoomToFit: Incomplete | None = ...,
        pageMargins: Incomplete | None = ...,
        pageSetup: Incomplete | None = ...,
        headerFooter: Incomplete | None = ...,
    ) -> None: ...

class CustomChartsheetViews(Serialisable):
    tagname: str
    customSheetView: Incomplete
    __elements__: Incomplete
    def __init__(self, customSheetView: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class ErrorBars(Serialisable):
    tagname: str
    errDir: Incomplete
    direction: Incomplete
    errBarType: Incomplete
    style: Incomplete
    errValType: Incomplete
    size: Incomplete
    noEndCap: Incomplete
    plus: Incomplete
    minus: Incomplete
    val: Incomplete
    spPr: Incomplete
    graphicalProperties: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        errDir: Incomplete | None = ...,
        errBarType: str = ...,
        errValType: str = ...,
        noEndCap: Incomplete | None = ...,
        plus: Incomplete | None = ...,
        minus: Incomplete | None = ...,
        val: Incomplete | None = ...,
        spPr: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class EmbeddedWAVAudioFile(Serialisable):  # type: ignore[misc]
    name: Incomplete
    def __init__(self, name: Incomplete | None = ...) -> None: ...

class Hyperlink(Serialisable):
    tagname: str
    namespace: Incomplete
    invalidUrl: Incomplete
    action: Incomplete
    tgtFrame: Incomplete
    tooltip: Incomplete
    history: Incomplete
    highlightClick: Incomplete
    endSnd: Incomplete
    snd: Incomplete
    extLst: Incomplete
    id: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        invalidUrl: Incomplete | None = ...,
        action: Incomplete | None = ...,
        tgtFrame: Incomplete | None = ...,
        tooltip: Incomplete | None = ...,
        history: Incomplete | None = ...,
        highlightClick: Incomplete | None = ...,
        endSnd: Incomplete | None = ...,
        snd: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        id: Incomplete | None = ...,
    ) -> None: ...

class Font(Serialisable):
    tagname: str
    namespace: Incomplete
    typeface: Incomplete
    panose: Incomplete
    pitchFamily: Incomplete
    charset: Incomplete
    def __init__(
        self,
        typeface: Incomplete | None = ...,
        panose: Incomplete | None = ...,
        pitchFamily: Incomplete | None = ...,
        charset: Incomplete | None = ...,
    ) -> None: ...

class CharacterProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    kumimoji: Incomplete
    lang: Incomplete
    altLang: Incomplete
    sz: Incomplete
    b: Incomplete
    i: Incomplete
    u: Incomplete
    strike: Incomplete
    kern: Incomplete
    cap: Incomplete
    spc: Incomplete
    normalizeH: Incomplete
    baseline: Incomplete
    noProof: Incomplete
    dirty: Incomplete
    err: Incomplete
    smtClean: Incomplete
    smtId: Incomplete
    bmk: Incomplete
    ln: Incomplete
    highlight: Incomplete
    latin: Incomplete
    ea: Incomplete
    cs: Incomplete
    sym: Incomplete
    hlinkClick: Incomplete
    hlinkMouseOver: Incomplete
    rtl: Incomplete
    extLst: Incomplete
    noFill: Incomplete
    solidFill: Incomplete
    gradFill: Incomplete
    blipFill: Incomplete
    pattFill: Incomplete
    grpFill: Incomplete
    effectLst: Incomplete
    effectDag: Incomplete
    uLnTx: Incomplete
    uLn: Incomplete
    uFillTx: Incomplete
    uFill: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        kumimoji: Incomplete | None = ...,
        lang: Incomplete | None = ...,
        altLang: Incomplete | None = ...,
        sz: Incomplete | None = ...,
        b: Incomplete | None = ...,
        i: Incomplete | None = ...,
        u: Incomplete | None = ...,
        strike: Incomplete | None = ...,
        kern: Incomplete | None = ...,
        cap: Incomplete | None = ...,
        spc: Incomplete | None = ...,
        normalizeH: Incomplete | None = ...,
        baseline: Incomplete | None = ...,
        noProof: Incomplete | None = ...,
        dirty: Incomplete | None = ...,
        err: Incomplete | None = ...,
        smtClean: Incomplete | None = ...,
        smtId: Incomplete | None = ...,
        bmk: Incomplete | None = ...,
        ln: Incomplete | None = ...,
        highlight: Incomplete | None = ...,
        latin: Incomplete | None = ...,
        ea: Incomplete | None = ...,
        cs: Incomplete | None = ...,
        sym: Incomplete | None = ...,
        hlinkClick: Incomplete | None = ...,
        hlinkMouseOver: Incomplete | None = ...,
        rtl: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        noFill: Incomplete | None = ...,
        solidFill: Incomplete | None = ...,
        gradFill: Incomplete | None = ...,
        blipFill: Incomplete | None = ...,
        pattFill: Incomplete | None = ...,
        grpFill: Incomplete | None = ...,
        effectLst: Incomplete | None = ...,
        effectDag: Incomplete | None = ...,
        uLnTx: Incomplete | None = ...,
        uLn: Incomplete | None = ...,
        uFillTx: Incomplete | None = ...,
        uFill: Incomplete | None = ...,
    ) -> None: ...

class TabStop(Serialisable):  # type: ignore[misc]
    pos: Incomplete
    algn: Incomplete
    def __init__(self, pos: Incomplete | None = ..., algn: Incomplete | None = ...) -> None: ...

class TabStopList(Serialisable):  # type: ignore[misc]
    tab: Incomplete
    def __init__(self, tab: Incomplete | None = ...) -> None: ...

class Spacing(Serialisable):
    spcPct: Incomplete
    spcPts: Incomplete
    __elements__: Incomplete
    def __init__(self, spcPct: Incomplete | None = ..., spcPts: Incomplete | None = ...) -> None: ...

class AutonumberBullet(Serialisable):
    type: Incomplete
    startAt: Incomplete
    def __init__(self, type: Incomplete | None = ..., startAt: Incomplete | None = ...) -> None: ...

class ParagraphProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    marL: Incomplete
    marR: Incomplete
    lvl: Incomplete
    indent: Incomplete
    algn: Incomplete
    defTabSz: Incomplete
    rtl: Incomplete
    eaLnBrk: Incomplete
    fontAlgn: Incomplete
    latinLnBrk: Incomplete
    hangingPunct: Incomplete
    lnSpc: Incomplete
    spcBef: Incomplete
    spcAft: Incomplete
    tabLst: Incomplete
    defRPr: Incomplete
    extLst: Incomplete
    buClrTx: Incomplete
    buClr: Incomplete
    buSzTx: Incomplete
    buSzPct: Incomplete
    buSzPts: Incomplete
    buFontTx: Incomplete
    buFont: Incomplete
    buNone: Incomplete
    buAutoNum: Incomplete
    buChar: Incomplete
    buBlip: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        marL: Incomplete | None = ...,
        marR: Incomplete | None = ...,
        lvl: Incomplete | None = ...,
        indent: Incomplete | None = ...,
        algn: Incomplete | None = ...,
        defTabSz: Incomplete | None = ...,
        rtl: Incomplete | None = ...,
        eaLnBrk: Incomplete | None = ...,
        fontAlgn: Incomplete | None = ...,
        latinLnBrk: Incomplete | None = ...,
        hangingPunct: Incomplete | None = ...,
        lnSpc: Incomplete | None = ...,
        spcBef: Incomplete | None = ...,
        spcAft: Incomplete | None = ...,
        tabLst: Incomplete | None = ...,
        defRPr: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        buClrTx: Incomplete | None = ...,
        buClr: Incomplete | None = ...,
        buSzTx: Incomplete | None = ...,
        buSzPct: Incomplete | None = ...,
        buSzPts: Incomplete | None = ...,
        buFontTx: Incomplete | None = ...,
        buFont: Incomplete | None = ...,
        buNone: Incomplete | None = ...,
        buAutoNum: Incomplete | None = ...,
        buChar: Incomplete | None = ...,
        buBlip: Incomplete | None = ...,
    ) -> None: ...

class ListStyle(Serialisable):
    tagname: str
    namespace: Incomplete
    defPPr: Incomplete
    lvl1pPr: Incomplete
    lvl2pPr: Incomplete
    lvl3pPr: Incomplete
    lvl4pPr: Incomplete
    lvl5pPr: Incomplete
    lvl6pPr: Incomplete
    lvl7pPr: Incomplete
    lvl8pPr: Incomplete
    lvl9pPr: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        defPPr: Incomplete | None = ...,
        lvl1pPr: Incomplete | None = ...,
        lvl2pPr: Incomplete | None = ...,
        lvl3pPr: Incomplete | None = ...,
        lvl4pPr: Incomplete | None = ...,
        lvl5pPr: Incomplete | None = ...,
        lvl6pPr: Incomplete | None = ...,
        lvl7pPr: Incomplete | None = ...,
        lvl8pPr: Incomplete | None = ...,
        lvl9pPr: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class RegularTextRun(Serialisable):
    tagname: str
    namespace: Incomplete
    rPr: Incomplete
    properties: Incomplete
    t: Incomplete
    value: Incomplete
    __elements__: Incomplete
    def __init__(self, rPr: Incomplete | None = ..., t: str = ...) -> None: ...

class LineBreak(Serialisable):
    tagname: str
    namespace: Incomplete
    rPr: Incomplete
    __elements__: Incomplete
    def __init__(self, rPr: Incomplete | None = ...) -> None: ...

class TextField(Serialisable):
    id: Incomplete
    type: Incomplete
    rPr: Incomplete
    pPr: Incomplete
    t: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        id: Incomplete | None = ...,
        type: Incomplete | None = ...,
        rPr: Incomplete | None = ...,
        pPr: Incomplete | None = ...,
        t: Incomplete | None = ...,
    ) -> None: ...

class Paragraph(Serialisable):
    tagname: str
    namespace: Incomplete
    pPr: Incomplete
    properties: Incomplete
    endParaRPr: Incomplete
    r: Incomplete
    text: Incomplete
    br: Incomplete
    fld: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        pPr: Incomplete | None = ...,
        endParaRPr: Incomplete | None = ...,
        r: Incomplete | None = ...,
        br: Incomplete | None = ...,
        fld: Incomplete | None = ...,
    ) -> None: ...

class GeomGuide(Serialisable):
    name: Incomplete
    fmla: Incomplete
    def __init__(self, name: Incomplete | None = ..., fmla: Incomplete | None = ...) -> None: ...

class GeomGuideList(Serialisable):
    gd: Incomplete
    def __init__(self, gd: Incomplete | None = ...) -> None: ...

class PresetTextShape(Serialisable):
    prst: Incomplete
    avLst: Incomplete
    def __init__(self, prst: Incomplete | None = ..., avLst: Incomplete | None = ...) -> None: ...

class TextNormalAutofit(Serialisable):
    fontScale: Incomplete
    lnSpcReduction: Incomplete
    def __init__(self, fontScale: Incomplete | None = ..., lnSpcReduction: Incomplete | None = ...) -> None: ...

class RichTextProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    rot: Incomplete
    spcFirstLastPara: Incomplete
    vertOverflow: Incomplete
    horzOverflow: Incomplete
    vert: Incomplete
    wrap: Incomplete
    lIns: Incomplete
    tIns: Incomplete
    rIns: Incomplete
    bIns: Incomplete
    numCol: Incomplete
    spcCol: Incomplete
    rtlCol: Incomplete
    fromWordArt: Incomplete
    anchor: Incomplete
    anchorCtr: Incomplete
    forceAA: Incomplete
    upright: Incomplete
    compatLnSpc: Incomplete
    prstTxWarp: Incomplete
    scene3d: Incomplete
    extLst: Incomplete
    noAutofit: Incomplete
    normAutofit: Incomplete
    spAutoFit: Incomplete
    flatTx: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        rot: Incomplete | None = ...,
        spcFirstLastPara: Incomplete | None = ...,
        vertOverflow: Incomplete | None = ...,
        horzOverflow: Incomplete | None = ...,
        vert: Incomplete | None = ...,
        wrap: Incomplete | None = ...,
        lIns: Incomplete | None = ...,
        tIns: Incomplete | None = ...,
        rIns: Incomplete | None = ...,
        bIns: Incomplete | None = ...,
        numCol: Incomplete | None = ...,
        spcCol: Incomplete | None = ...,
        rtlCol: Incomplete | None = ...,
        fromWordArt: Incomplete | None = ...,
        anchor: Incomplete | None = ...,
        anchorCtr: Incomplete | None = ...,
        forceAA: Incomplete | None = ...,
        upright: Incomplete | None = ...,
        compatLnSpc: Incomplete | None = ...,
        prstTxWarp: Incomplete | None = ...,
        scene3d: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        noAutofit: Incomplete | None = ...,
        normAutofit: Incomplete | None = ...,
        spAutoFit: Incomplete | None = ...,
        flatTx: Incomplete | None = ...,
    ) -> None: ...



from .alignment import Alignment as Alignment
from .borders import Border as Border, Side as Side
from .colors import Color as Color
from .fills import Fill as Fill, GradientFill as GradientFill, PatternFill as PatternFill
from .fonts import DEFAULT_FONT as DEFAULT_FONT, Font as Font
from .named_styles import NamedStyle as NamedStyle
from .numbers import NumberFormatDescriptor as NumberFormatDescriptor, is_builtin as is_builtin, is_date_format as is_date_format
from .protection import Protection as Protection



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class _Protected:
    def set_password(self, value: str = ..., already_hashed: bool = ...) -> None: ...
    @property
    def password(self): ...
    @password.setter
    def password(self, value) -> None: ...

class SheetProtection(Serialisable, _Protected):
    tagname: str
    sheet: Incomplete
    enabled: Incomplete
    objects: Incomplete
    scenarios: Incomplete
    formatCells: Incomplete
    formatColumns: Incomplete
    formatRows: Incomplete
    insertColumns: Incomplete
    insertRows: Incomplete
    insertHyperlinks: Incomplete
    deleteColumns: Incomplete
    deleteRows: Incomplete
    selectLockedCells: Incomplete
    selectUnlockedCells: Incomplete
    sort: Incomplete
    autoFilter: Incomplete
    pivotTables: Incomplete
    saltValue: Incomplete
    spinCount: Incomplete
    algorithmName: Incomplete
    hashValue: Incomplete
    __attrs__: Incomplete
    password: Incomplete
    def __init__(
        self,
        sheet: bool = ...,
        objects: bool = ...,
        scenarios: bool = ...,
        formatCells: bool = ...,
        formatRows: bool = ...,
        formatColumns: bool = ...,
        insertColumns: bool = ...,
        insertRows: bool = ...,
        insertHyperlinks: bool = ...,
        deleteColumns: bool = ...,
        deleteRows: bool = ...,
        selectLockedCells: bool = ...,
        selectUnlockedCells: bool = ...,
        sort: bool = ...,
        autoFilter: bool = ...,
        pivotTables: bool = ...,
        password: Incomplete | None = ...,
        algorithmName: Incomplete | None = ...,
        saltValue: Incomplete | None = ...,
        spinCount: Incomplete | None = ...,
        hashValue: Incomplete | None = ...,
    ) -> None: ...
    def set_password(self, value: str = ..., already_hashed: bool = ...) -> None: ...
    def enable(self) -> None: ...
    def disable(self) -> None: ...
    def __bool__(self) -> bool: ...



from _typeshed import Incomplete

from openpyxl.descriptors import String
from openpyxl.descriptors.serialisable import Serialisable

TABLESTYLES: Incomplete
PIVOTSTYLES: Incomplete

class TableStyleInfo(Serialisable):
    tagname: str
    name: Incomplete
    showFirstColumn: Incomplete
    showLastColumn: Incomplete
    showRowStripes: Incomplete
    showColumnStripes: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        showFirstColumn: Incomplete | None = ...,
        showLastColumn: Incomplete | None = ...,
        showRowStripes: Incomplete | None = ...,
        showColumnStripes: Incomplete | None = ...,
    ) -> None: ...

class XMLColumnProps(Serialisable):
    tagname: str
    mapId: Incomplete
    xpath: Incomplete
    denormalized: Incomplete
    xmlDataType: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        mapId: Incomplete | None = ...,
        xpath: Incomplete | None = ...,
        denormalized: Incomplete | None = ...,
        xmlDataType: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class TableFormula(Serialisable):
    tagname: str
    array: Incomplete
    attr_text: Incomplete
    text: Incomplete
    def __init__(self, array: Incomplete | None = ..., attr_text: Incomplete | None = ...) -> None: ...

class TableColumn(Serialisable):
    tagname: str
    id: Incomplete
    uniqueName: Incomplete
    name: Incomplete
    totalsRowFunction: Incomplete
    totalsRowLabel: Incomplete
    queryTableFieldId: Incomplete
    headerRowDxfId: Incomplete
    dataDxfId: Incomplete
    totalsRowDxfId: Incomplete
    headerRowCellStyle: Incomplete
    dataCellStyle: Incomplete
    totalsRowCellStyle: Incomplete
    calculatedColumnFormula: Incomplete
    totalsRowFormula: Incomplete
    xmlColumnPr: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        id: Incomplete | None = ...,
        uniqueName: Incomplete | None = ...,
        name: Incomplete | None = ...,
        totalsRowFunction: Incomplete | None = ...,
        totalsRowLabel: Incomplete | None = ...,
        queryTableFieldId: Incomplete | None = ...,
        headerRowDxfId: Incomplete | None = ...,
        dataDxfId: Incomplete | None = ...,
        totalsRowDxfId: Incomplete | None = ...,
        headerRowCellStyle: Incomplete | None = ...,
        dataCellStyle: Incomplete | None = ...,
        totalsRowCellStyle: Incomplete | None = ...,
        calculatedColumnFormula: Incomplete | None = ...,
        totalsRowFormula: Incomplete | None = ...,
        xmlColumnPr: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...
    def __iter__(self): ...
    @classmethod
    def from_tree(cls, node): ...

class TableNameDescriptor(String):
    def __set__(self, instance, value) -> None: ...

class Table(Serialisable):
    mime_type: str
    tagname: str
    id: Incomplete
    name: Incomplete
    displayName: Incomplete
    comment: Incomplete
    ref: Incomplete
    tableType: Incomplete
    headerRowCount: Incomplete
    insertRow: Incomplete
    insertRowShift: Incomplete
    totalsRowCount: Incomplete
    totalsRowShown: Incomplete
    published: Incomplete
    headerRowDxfId: Incomplete
    dataDxfId: Incomplete
    totalsRowDxfId: Incomplete
    headerRowBorderDxfId: Incomplete
    tableBorderDxfId: Incomplete
    totalsRowBorderDxfId: Incomplete
    headerRowCellStyle: Incomplete
    dataCellStyle: Incomplete
    totalsRowCellStyle: Incomplete
    connectionId: Incomplete
    autoFilter: Incomplete
    sortState: Incomplete
    tableColumns: Incomplete
    tableStyleInfo: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        id: int = ...,
        displayName: Incomplete | None = ...,
        ref: Incomplete | None = ...,
        name: Incomplete | None = ...,
        comment: Incomplete | None = ...,
        tableType: Incomplete | None = ...,
        headerRowCount: int = ...,
        insertRow: Incomplete | None = ...,
        insertRowShift: Incomplete | None = ...,
        totalsRowCount: Incomplete | None = ...,
        totalsRowShown: Incomplete | None = ...,
        published: Incomplete | None = ...,
        headerRowDxfId: Incomplete | None = ...,
        dataDxfId: Incomplete | None = ...,
        totalsRowDxfId: Incomplete | None = ...,
        headerRowBorderDxfId: Incomplete | None = ...,
        tableBorderDxfId: Incomplete | None = ...,
        totalsRowBorderDxfId: Incomplete | None = ...,
        headerRowCellStyle: Incomplete | None = ...,
        dataCellStyle: Incomplete | None = ...,
        totalsRowCellStyle: Incomplete | None = ...,
        connectionId: Incomplete | None = ...,
        autoFilter: Incomplete | None = ...,
        sortState: Incomplete | None = ...,
        tableColumns=...,
        tableStyleInfo: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...
    def to_tree(self): ...
    @property
    def path(self): ...
    @property
    def column_names(self): ...

class TablePartList(Serialisable):
    tagname: str
    # Overwritten by property below
    # count: Integer
    tablePart: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(self, count: Incomplete | None = ..., tablePart=...) -> None: ...
    def append(self, part) -> None: ...
    @property
    def count(self): ...
    def __bool__(self) -> bool: ...

class TableList(dict[Incomplete, Incomplete]):
    def add(self, table) -> None: ...
    def get(self, name: Incomplete | None = ..., table_range: Incomplete | None = ...): ...
    def items(self): ...



from _typeshed import Incomplete

from . import Integer, MatchPattern, MinMax, String
from .serialisable import Serialisable

class HexBinary(MatchPattern):
    pattern: str

class UniversalMeasure(MatchPattern):
    pattern: str

class TextPoint(MinMax):
    expected_type: Incomplete
    min: int
    max: int

Coordinate = Integer

class Percentage(MinMax):
    pattern: str
    min: int
    max: int
    def __set__(self, instance, value) -> None: ...

class Extension(Serialisable):
    uri: Incomplete
    def __init__(self, uri: Incomplete | None = ...) -> None: ...

class ExtensionList(Serialisable):
    ext: Incomplete
    def __init__(self, ext=...) -> None: ...

class Relation(String):
    namespace: Incomplete
    allow_none: bool

class Base64Binary(MatchPattern):
    pattern: str

class Guid(MatchPattern):
    pattern: str

class CellRange(MatchPattern):
    pattern: str
    allow_none: bool
    def __set__(self, instance, value) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class BookView(Serialisable):
    tagname: str
    visibility: Incomplete
    minimized: Incomplete
    showHorizontalScroll: Incomplete
    showVerticalScroll: Incomplete
    showSheetTabs: Incomplete
    xWindow: Incomplete
    yWindow: Incomplete
    windowWidth: Incomplete
    windowHeight: Incomplete
    tabRatio: Incomplete
    firstSheet: Incomplete
    activeTab: Incomplete
    autoFilterDateGrouping: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        visibility: str = ...,
        minimized: bool = ...,
        showHorizontalScroll: bool = ...,
        showVerticalScroll: bool = ...,
        showSheetTabs: bool = ...,
        xWindow: Incomplete | None = ...,
        yWindow: Incomplete | None = ...,
        windowWidth: Incomplete | None = ...,
        windowHeight: Incomplete | None = ...,
        tabRatio: int = ...,
        firstSheet: int = ...,
        activeTab: int = ...,
        autoFilterDateGrouping: bool = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class CustomWorkbookView(Serialisable):
    tagname: str
    name: Incomplete
    guid: Incomplete
    autoUpdate: Incomplete
    mergeInterval: Incomplete
    changesSavedWin: Incomplete
    onlySync: Incomplete
    personalView: Incomplete
    includePrintSettings: Incomplete
    includeHiddenRowCol: Incomplete
    maximized: Incomplete
    minimized: Incomplete
    showHorizontalScroll: Incomplete
    showVerticalScroll: Incomplete
    showSheetTabs: Incomplete
    xWindow: Incomplete
    yWindow: Incomplete
    windowWidth: Incomplete
    windowHeight: Incomplete
    tabRatio: Incomplete
    activeSheetId: Incomplete
    showFormulaBar: Incomplete
    showStatusbar: Incomplete
    showComments: Incomplete
    showObjects: Incomplete
    extLst: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        name: Incomplete | None = ...,
        guid: Incomplete | None = ...,
        autoUpdate: Incomplete | None = ...,
        mergeInterval: Incomplete | None = ...,
        changesSavedWin: Incomplete | None = ...,
        onlySync: Incomplete | None = ...,
        personalView: Incomplete | None = ...,
        includePrintSettings: Incomplete | None = ...,
        includeHiddenRowCol: Incomplete | None = ...,
        maximized: Incomplete | None = ...,
        minimized: Incomplete | None = ...,
        showHorizontalScroll: Incomplete | None = ...,
        showVerticalScroll: Incomplete | None = ...,
        showSheetTabs: Incomplete | None = ...,
        xWindow: Incomplete | None = ...,
        yWindow: Incomplete | None = ...,
        windowWidth: Incomplete | None = ...,
        windowHeight: Incomplete | None = ...,
        tabRatio: Incomplete | None = ...,
        activeSheetId: Incomplete | None = ...,
        showFormulaBar: Incomplete | None = ...,
        showStatusbar: Incomplete | None = ...,
        showComments: str = ...,
        showObjects: str = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class SheetBackgroundPicture(Serialisable):
    tagname: str
    id: Incomplete
    def __init__(self, id) -> None: ...

class DrawingHF(Serialisable):
    id: Incomplete
    lho: Incomplete
    leftHeaderOddPages: Incomplete
    lhe: Incomplete
    leftHeaderEvenPages: Incomplete
    lhf: Incomplete
    leftHeaderFirstPage: Incomplete
    cho: Incomplete
    centerHeaderOddPages: Incomplete
    che: Incomplete
    centerHeaderEvenPages: Incomplete
    chf: Incomplete
    centerHeaderFirstPage: Incomplete
    rho: Incomplete
    rightHeaderOddPages: Incomplete
    rhe: Incomplete
    rightHeaderEvenPages: Incomplete
    rhf: Incomplete
    rightHeaderFirstPage: Incomplete
    lfo: Incomplete
    leftFooterOddPages: Incomplete
    lfe: Incomplete
    leftFooterEvenPages: Incomplete
    lff: Incomplete
    leftFooterFirstPage: Incomplete
    cfo: Incomplete
    centerFooterOddPages: Incomplete
    cfe: Incomplete
    centerFooterEvenPages: Incomplete
    cff: Incomplete
    centerFooterFirstPage: Incomplete
    rfo: Incomplete
    rightFooterOddPages: Incomplete
    rfe: Incomplete
    rightFooterEvenPages: Incomplete
    rff: Incomplete
    rightFooterFirstPage: Incomplete
    def __init__(
        self,
        id: Incomplete | None = ...,
        lho: Incomplete | None = ...,
        lhe: Incomplete | None = ...,
        lhf: Incomplete | None = ...,
        cho: Incomplete | None = ...,
        che: Incomplete | None = ...,
        chf: Incomplete | None = ...,
        rho: Incomplete | None = ...,
        rhe: Incomplete | None = ...,
        rhf: Incomplete | None = ...,
        lfo: Incomplete | None = ...,
        lfe: Incomplete | None = ...,
        lff: Incomplete | None = ...,
        cfo: Incomplete | None = ...,
        cfe: Incomplete | None = ...,
        cff: Incomplete | None = ...,
        rfo: Incomplete | None = ...,
        rfe: Incomplete | None = ...,
        rff: Incomplete | None = ...,
    ) -> None: ...



from _typeshed import Incomplete

from .base import Bool, Convertible, Descriptor, Float, Integer, MinMax, NoneSet, Set, String

class Nested(Descriptor):
    nested: bool
    attribute: str
    def __set__(self, instance, value) -> None: ...
    def from_tree(self, node): ...
    def to_tree(self, tagname: Incomplete | None = ..., value: Incomplete | None = ..., namespace: Incomplete | None = ...): ...

class NestedValue(Nested, Convertible): ...

class NestedText(NestedValue):
    def from_tree(self, node): ...
    def to_tree(self, tagname: Incomplete | None = ..., value: Incomplete | None = ..., namespace: Incomplete | None = ...): ...

class NestedFloat(NestedValue, Float): ...
class NestedInteger(NestedValue, Integer): ...
class NestedString(NestedValue, String): ...

class NestedBool(NestedValue, Bool):
    def from_tree(self, node): ...

class NestedNoneSet(Nested, NoneSet): ...
class NestedSet(Nested, Set): ...
class NestedMinMax(Nested, MinMax): ...

class EmptyTag(Nested, Bool):
    def from_tree(self, node): ...
    def to_tree(self, tagname: Incomplete | None = ..., value: Incomplete | None = ..., namespace: Incomplete | None = ...): ...



from _typeshed import Incomplete

from openpyxl.descriptors import Typed
from openpyxl.descriptors.nested import NestedMinMax

class NestedGapAmount(NestedMinMax):
    allow_none: bool
    min: int
    max: int

class NestedOverlap(NestedMinMax):
    allow_none: bool
    min: int
    max: int

class NumberFormatDescriptor(Typed):
    expected_type: Incomplete
    allow_none: bool
    def __set__(self, instance, value) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class FunctionGroup(Serialisable):
    tagname: str
    name: Incomplete
    def __init__(self, name: Incomplete | None = ...) -> None: ...

class FunctionGroupList(Serialisable):
    tagname: str
    builtInGroupCount: Incomplete
    functionGroup: Incomplete
    __elements__: Incomplete
    def __init__(self, builtInGroupCount: int = ..., functionGroup=...) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class SmartTag(Serialisable):
    tagname: str
    namespaceUri: Incomplete
    name: Incomplete
    url: Incomplete
    def __init__(
        self, namespaceUri: Incomplete | None = ..., name: Incomplete | None = ..., url: Incomplete | None = ...
    ) -> None: ...

class SmartTagList(Serialisable):
    tagname: str
    smartTagType: Incomplete
    __elements__: Incomplete
    def __init__(self, smartTagType=...) -> None: ...

class SmartTagProperties(Serialisable):
    tagname: str
    embed: Incomplete
    show: Incomplete
    def __init__(self, embed: Incomplete | None = ..., show: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class CustomProperty(Serialisable):
    tagname: str
    name: Incomplete
    def __init__(self, name: Incomplete | None = ...) -> None: ...

class CustomProperties(Serialisable):
    tagname: str
    customPr: Incomplete
    __elements__: Incomplete
    def __init__(self, customPr=...) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class GraphicFrameLocking(Serialisable):
    noGrp: Incomplete
    noDrilldown: Incomplete
    noSelect: Incomplete
    noChangeAspect: Incomplete
    noMove: Incomplete
    noResize: Incomplete
    extLst: Incomplete
    def __init__(
        self,
        noGrp: Incomplete | None = ...,
        noDrilldown: Incomplete | None = ...,
        noSelect: Incomplete | None = ...,
        noChangeAspect: Incomplete | None = ...,
        noMove: Incomplete | None = ...,
        noResize: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
    ) -> None: ...

class NonVisualGraphicFrameProperties(Serialisable):
    tagname: str
    graphicFrameLocks: Incomplete
    extLst: Incomplete
    def __init__(self, graphicFrameLocks: Incomplete | None = ..., extLst: Incomplete | None = ...) -> None: ...

class NonVisualGraphicFrame(Serialisable):
    tagname: str
    cNvPr: Incomplete
    cNvGraphicFramePr: Incomplete
    __elements__: Incomplete
    def __init__(self, cNvPr: Incomplete | None = ..., cNvGraphicFramePr: Incomplete | None = ...) -> None: ...

class GraphicData(Serialisable):
    tagname: str
    namespace: Incomplete
    uri: Incomplete
    chart: Incomplete
    def __init__(self, uri=..., chart: Incomplete | None = ...) -> None: ...

class GraphicObject(Serialisable):
    tagname: str
    namespace: Incomplete
    graphicData: Incomplete
    def __init__(self, graphicData: Incomplete | None = ...) -> None: ...

class GraphicFrame(Serialisable):
    tagname: str
    nvGraphicFramePr: Incomplete
    xfrm: Incomplete
    graphic: Incomplete
    macro: Incomplete
    fPublished: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        nvGraphicFramePr: Incomplete | None = ...,
        xfrm: Incomplete | None = ...,
        graphic: Incomplete | None = ...,
        macro: Incomplete | None = ...,
        fPublished: Incomplete | None = ...,
    ) -> None: ...

class GroupShape(Serialisable):
    nvGrpSpPr: Incomplete
    nonVisualProperties: Incomplete
    grpSpPr: Incomplete
    visualProperties: Incomplete
    pic: Incomplete
    __elements__: Incomplete
    def __init__(
        self, nvGrpSpPr: Incomplete | None = ..., grpSpPr: Incomplete | None = ..., pic: Incomplete | None = ...
    ) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

horizontal_alignments: Incomplete
vertical_aligments: Incomplete

class Alignment(Serialisable):
    tagname: str
    __fields__: Incomplete
    horizontal: Incomplete
    vertical: Incomplete
    textRotation: Incomplete
    text_rotation: Incomplete
    wrapText: Incomplete
    wrap_text: Incomplete
    shrinkToFit: Incomplete
    shrink_to_fit: Incomplete
    indent: Incomplete
    relativeIndent: Incomplete
    justifyLastLine: Incomplete
    readingOrder: Incomplete
    def __init__(
        self,
        horizontal: Incomplete | None = ...,
        vertical: Incomplete | None = ...,
        textRotation: int = ...,
        wrapText: Incomplete | None = ...,
        shrinkToFit: Incomplete | None = ...,
        indent: int = ...,
        relativeIndent: int = ...,
        justifyLastLine: Incomplete | None = ...,
        readingOrder: int = ...,
        text_rotation: Incomplete | None = ...,
        wrap_text: Incomplete | None = ...,
        shrink_to_fit: Incomplete | None = ...,
        mergeCell: Incomplete | None = ...,
    ) -> None: ...
    def __iter__(self): ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class PatternFillProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    prst: Incomplete
    preset: Incomplete
    fgClr: Incomplete
    foreground: Incomplete
    bgClr: Incomplete
    background: Incomplete
    __elements__: Incomplete
    def __init__(self, prst: Incomplete | None = ..., fgClr: Incomplete | None = ..., bgClr: Incomplete | None = ...) -> None: ...

class RelativeRect(Serialisable):
    tagname: str
    namespace: Incomplete
    l: Incomplete
    left: Incomplete
    t: Incomplete
    top: Incomplete
    r: Incomplete
    right: Incomplete
    b: Incomplete
    bottom: Incomplete
    def __init__(
        self, l: Incomplete | None = ..., t: Incomplete | None = ..., r: Incomplete | None = ..., b: Incomplete | None = ...
    ) -> None: ...

class StretchInfoProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    fillRect: Incomplete
    def __init__(self, fillRect=...) -> None: ...

class GradientStop(Serialisable):
    tagname: str
    namespace: Incomplete
    pos: Incomplete
    scrgbClr: Incomplete
    RGBPercent: Incomplete
    srgbClr: Incomplete
    RGB: Incomplete
    hslClr: Incomplete
    sysClr: Incomplete
    schemeClr: Incomplete
    prstClr: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        pos: Incomplete | None = ...,
        scrgbClr: Incomplete | None = ...,
        srgbClr: Incomplete | None = ...,
        hslClr: Incomplete | None = ...,
        sysClr: Incomplete | None = ...,
        schemeClr: Incomplete | None = ...,
        prstClr: Incomplete | None = ...,
    ) -> None: ...

class LinearShadeProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    ang: Incomplete
    scaled: Incomplete
    def __init__(self, ang: Incomplete | None = ..., scaled: Incomplete | None = ...) -> None: ...

class PathShadeProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    path: Incomplete
    fillToRect: Incomplete
    def __init__(self, path: Incomplete | None = ..., fillToRect: Incomplete | None = ...) -> None: ...

class GradientFillProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    flip: Incomplete
    rotWithShape: Incomplete
    gsLst: Incomplete
    stop_list: Incomplete
    lin: Incomplete
    linear: Incomplete
    path: Incomplete
    tileRect: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        flip: Incomplete | None = ...,
        rotWithShape: Incomplete | None = ...,
        gsLst=...,
        lin: Incomplete | None = ...,
        path: Incomplete | None = ...,
        tileRect: Incomplete | None = ...,
    ) -> None: ...

class SolidColorFillProperties(Serialisable):
    tagname: str
    scrgbClr: Incomplete
    RGBPercent: Incomplete
    srgbClr: Incomplete
    RGB: Incomplete
    hslClr: Incomplete
    sysClr: Incomplete
    schemeClr: Incomplete
    prstClr: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        scrgbClr: Incomplete | None = ...,
        srgbClr: Incomplete | None = ...,
        hslClr: Incomplete | None = ...,
        sysClr: Incomplete | None = ...,
        schemeClr: Incomplete | None = ...,
        prstClr: Incomplete | None = ...,
    ) -> None: ...

class Blip(Serialisable):
    tagname: str
    namespace: Incomplete
    cstate: Incomplete
    embed: Incomplete
    link: Incomplete
    noGrp: Incomplete
    noSelect: Incomplete
    noRot: Incomplete
    noChangeAspect: Incomplete
    noMove: Incomplete
    noResize: Incomplete
    noEditPoints: Incomplete
    noAdjustHandles: Incomplete
    noChangeArrowheads: Incomplete
    noChangeShapeType: Incomplete
    extLst: Incomplete
    alphaBiLevel: Incomplete
    alphaCeiling: Incomplete
    alphaFloor: Incomplete
    alphaInv: Incomplete
    alphaMod: Incomplete
    alphaModFix: Incomplete
    alphaRepl: Incomplete
    biLevel: Incomplete
    blur: Incomplete
    clrChange: Incomplete
    clrRepl: Incomplete
    duotone: Incomplete
    fillOverlay: Incomplete
    grayscl: Incomplete
    hsl: Incomplete
    lum: Incomplete
    tint: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        cstate: Incomplete | None = ...,
        embed: Incomplete | None = ...,
        link: Incomplete | None = ...,
        noGrp: Incomplete | None = ...,
        noSelect: Incomplete | None = ...,
        noRot: Incomplete | None = ...,
        noChangeAspect: Incomplete | None = ...,
        noMove: Incomplete | None = ...,
        noResize: Incomplete | None = ...,
        noEditPoints: Incomplete | None = ...,
        noAdjustHandles: Incomplete | None = ...,
        noChangeArrowheads: Incomplete | None = ...,
        noChangeShapeType: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        alphaBiLevel: Incomplete | None = ...,
        alphaCeiling: Incomplete | None = ...,
        alphaFloor: Incomplete | None = ...,
        alphaInv: Incomplete | None = ...,
        alphaMod: Incomplete | None = ...,
        alphaModFix: Incomplete | None = ...,
        alphaRepl: Incomplete | None = ...,
        biLevel: Incomplete | None = ...,
        blur: Incomplete | None = ...,
        clrChange: Incomplete | None = ...,
        clrRepl: Incomplete | None = ...,
        duotone: Incomplete | None = ...,
        fillOverlay: Incomplete | None = ...,
        grayscl: Incomplete | None = ...,
        hsl: Incomplete | None = ...,
        lum: Incomplete | None = ...,
        tint: Incomplete | None = ...,
    ) -> None: ...

class TileInfoProperties(Serialisable):
    tx: Incomplete
    ty: Incomplete
    sx: Incomplete
    sy: Incomplete
    flip: Incomplete
    algn: Incomplete
    def __init__(
        self,
        tx: Incomplete | None = ...,
        ty: Incomplete | None = ...,
        sx: Incomplete | None = ...,
        sy: Incomplete | None = ...,
        flip: Incomplete | None = ...,
        algn: Incomplete | None = ...,
    ) -> None: ...

class BlipFillProperties(Serialisable):
    tagname: str
    dpi: Incomplete
    rotWithShape: Incomplete
    blip: Incomplete
    srcRect: Incomplete
    tile: Incomplete
    stretch: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        dpi: Incomplete | None = ...,
        rotWithShape: Incomplete | None = ...,
        blip: Incomplete | None = ...,
        tile: Incomplete | None = ...,
        stretch=...,
        srcRect: Incomplete | None = ...,
    ) -> None: ...



from _typeshed import Incomplete
from abc import abstractmethod

from ._3d import _3DBase
from ._chart import ChartBase

class _BarChartBase(ChartBase):
    barDir: Incomplete
    type: Incomplete
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    dataLabels: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        barDir: str = ...,
        grouping: str = ...,
        varyColors: Incomplete | None = ...,
        ser=...,
        dLbls: Incomplete | None = ...,
        **kw,
    ) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class BarChart(_BarChartBase):
    tagname: str
    barDir: Incomplete
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    gapWidth: Incomplete
    overlap: Incomplete
    serLines: Incomplete
    extLst: Incomplete
    x_axis: Incomplete
    y_axis: Incomplete
    __elements__: Incomplete
    legend: Incomplete
    def __init__(
        self,
        gapWidth: int = ...,
        overlap: Incomplete | None = ...,
        serLines: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        **kw,
    ) -> None: ...

class BarChart3D(_BarChartBase, _3DBase):
    tagname: str
    barDir: Incomplete
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    view3D: Incomplete
    floor: Incomplete
    sideWall: Incomplete
    backWall: Incomplete
    gapWidth: Incomplete
    gapDepth: Incomplete
    shape: Incomplete
    serLines: Incomplete
    extLst: Incomplete
    x_axis: Incomplete
    y_axis: Incomplete
    z_axis: Incomplete
    __elements__: Incomplete
    def __init__(
        self,
        gapWidth: int = ...,
        gapDepth: int = ...,
        shape: Incomplete | None = ...,
        serLines: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        **kw,
    ) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class Related(Serialisable):  # type: ignore[misc]
    id: Incomplete
    def __init__(self, id: Incomplete | None = ...) -> None: ...
    def to_tree(self, tagname, idx: Incomplete | None = ...): ...  # type: ignore[override]



from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.workbook.child import _WorkbookChild

class Chartsheet(_WorkbookChild, Serialisable):
    tagname: str
    mime_type: str
    sheetPr: Incomplete
    sheetViews: Incomplete
    sheetProtection: Incomplete
    customSheetViews: Incomplete
    pageMargins: Incomplete
    pageSetup: Incomplete
    drawing: Incomplete
    drawingHF: Incomplete
    picture: Incomplete
    webPublishItems: Incomplete
    extLst: Incomplete
    sheet_state: Incomplete
    headerFooter: Incomplete
    HeaderFooter: Incomplete
    __elements__: Incomplete
    __attrs__: Incomplete
    def __init__(
        self,
        sheetPr: Incomplete | None = ...,
        sheetViews: Incomplete | None = ...,
        sheetProtection: Incomplete | None = ...,
        customSheetViews: Incomplete | None = ...,
        pageMargins: Incomplete | None = ...,
        pageSetup: Incomplete | None = ...,
        headerFooter: Incomplete | None = ...,
        drawing: Incomplete | None = ...,
        drawingHF: Incomplete | None = ...,
        picture: Incomplete | None = ...,
        webPublishItems: Incomplete | None = ...,
        extLst: Incomplete | None = ...,
        parent: Incomplete | None = ...,
        title: str = ...,
        sheet_state: str = ...,
    ) -> None: ...
    def add_chart(self, chart) -> None: ...
    def to_tree(self): ...



from _typeshed import Incomplete
from collections.abc import Generator

from openpyxl.descriptors.serialisable import Serialisable

mimetypes: Incomplete

class FileExtension(Serialisable):
    tagname: str
    Extension: Incomplete
    ContentType: Incomplete
    def __init__(self, Extension, ContentType) -> None: ...

class Override(Serialisable):
    tagname: str
    PartName: Incomplete
    ContentType: Incomplete
    def __init__(self, PartName, ContentType) -> None: ...

DEFAULT_TYPES: Incomplete
DEFAULT_OVERRIDE: Incomplete

class Manifest(Serialisable):
    tagname: str
    Default: Incomplete
    Override: Incomplete
    path: str
    __elements__: Incomplete
    def __init__(self, Default=..., Override=...) -> None: ...
    @property
    def filenames(self): ...
    @property
    def extensions(self): ...
    def to_tree(self): ...
    def __contains__(self, content_type): ...
    def find(self, content_type): ...
    def findall(self, content_type) -> Generator[Incomplete, None, None]: ...
    def append(self, obj) -> None: ...
